#pragma once

#include "Zeolite_defines.h"

// app functions

void	zeoReportError(const char* lpErrorMessage);
void	zeoWriteToLog(const char* lpMessage);
void*	zeoGetHID();

// var functions

ZVAR	zvar_Create(long VarID);
ZVAR	zvar_CreateShared(long VarID, const char* lpVarName);
ZVAR	zvar_GetSharedVar(const char* lpVarName);
bool	zvar_Delete(ZVAR &hVar);
bool	zvar_ReInit(ZVAR hVar, long VarID);
bool	zvar_Rename(ZVAR hVar, const char* lpNewName);
const char*	zvar_GetName(ZVAR hVar);
long	zvar_GetTypeID(ZVAR hVar);
bool	zvar_IsType(ZVAR hVar, long VarID);
bool	zvar_GetValue(ZVAR hVar, void* pValue);
bool	zvar_SetValue(ZVAR hVar, void* pValue);
bool	zvar_GetValueEx(ZVAR hVar, long VarID, void* pValue);
bool	zvar_SetValueEx(ZVAR hVar, long VarID, void* pValue);
bool	zvar_GetValueText(ZVAR hVar, ZSTR hStrVar);
bool	zvar_SetValueText(ZVAR hVar, const char* lpValueText);
bool	zvar_CopyValue(ZVAR hVarDest, ZVAR hVarSrc);
bool	zvar_SetVarRef(ZVAR hPtrVar, ZVAR hSrcVar);
bool	zvar_EditUI(ZVAR hVar, const char* lpPrompt, unsigned long Flags, ZLIST hOptions);
void*	zvar_GetDataPtr(ZVAR hVar);

// var type functions

long	ztype_GetID(const char* lpTypeName);
bool	ztype_GetName(long VarID, ZSTR hTypeName);

// string functions

const char*	zstr_GetText(ZSTR hStr);
bool		zstr_SetText(ZSTR hStr, const char* lpText);

// list functions

long	zlist_nItems(ZLIST hList);
ZLIST	zlist_CreateItem(ZLIST hList, long VarID, const char* lpVarName);
ZLIST	zlist_GetItemI(ZLIST hList, long index);
ZLIST	zlist_GetItemA(ZLIST hList, const char* lpVarName);
bool	zlist_DelItemI(ZLIST hList, long index);
bool	zlist_DelItemA(ZLIST hList, const char* lpVarName);
bool	zlist_DelAll(ZLIST hList);
bool	zlist_SortI(ZLIST hList, long From, long To);
bool	zlist_SortA(ZLIST hList, const char* lpItemName, long To);
bool	zlist_Append(ZLIST hList, ZLIST hList2);

// map functions

bool	zmap_Init(ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag);
bool    zmap_Init2(ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, unsigned long AllocMode, long TileSize);
bool	zmap_Free(ZMAP hMap);
long	zmap_GetMapType(ZMAP hMap);
long	zmap_GetPixelSize(ZMAP hMap);
bool	zmap_GetWrapFlag(ZMAP hMap);
bool	zmap_SetWrapFlag(ZMAP hMap, bool WrapFlag);
bool	zmap_GetBusyFlag(ZMAP hMap);
bool	zmap_SetBusyFlag(ZMAP hMap, bool BusyFlag);
long	zmap_GetWidth(ZMAP hMap);
long	zmap_GetHeight(ZMAP hMap);
bool	zmap_GetPixel(ZMAP hMap, long x, long y, void* pValue);
bool	zmap_SetPixel(ZMAP hMap, long x, long y, void* pValue);
bool	zmap_GetMinMaxAlt(ZMAP hMap, float& minval, float& maxval);
bool	zmap_SetMinMaxAlt(ZMAP hMap, float& minval, float& maxval);
float	zmap_GetHorizScale(ZMAP hMap);
bool	zmap_SetHorizScale(ZMAP hMap, float scale);
bool	zmap_GetMosaicFlag(ZMAP hMap);
long	zmap_GetTileSize(ZMAP hMap);
long	zmap_GetTileCountX(ZMAP hMap);
long	zmap_GetTileCountY(ZMAP hMap);
bool	zmap_GetFlag(ZMAP hMap, long FlagID);
bool	zmap_SetFlag(ZMAP hMap, long FlagID, bool FlagValue);
unsigned long zmap_GetAllocMode(ZMAP hMap);
ZLIST	zmap_GetSettingsList(ZMAP hMap);
bool	zmap_SaveFile(ZMAP hMap, const char* lpFileName, ZFORMAT hFormat, bool SetFlags, bool ShowProgress);
bool	zmap_LoadFile(ZMAP hMap, const char* lpFileName, long MapTypeID, ZFORMAT hFormat, bool SetFlags, bool ShowProgress);
bool	zmap_GetNameInProject(ZMAP hMap, ZVAR hStr);
bool	zmap_InitMosaic(ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName);
bool	zmap_LoadMosaic(ZMAP hMap, const char* lpMosaicFileName);
bool	zmap_SaveMosaic(ZMAP hMap);
ZFORMAT zmap_GetFormat(ZMAP hMap);
bool	zmap_SetFormat(ZMAP hMap, ZFORMAT hFormat);
const char*	zmap_GetFilename(ZMAP hMap);
bool	zmap_CombineMosaic(ZMAP hMap);
bool	zmap_SplitToMosaic(ZMAP hMap, const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName);
bool	zmap_SaveMosaicAs(ZMAP hMap, const char* lpFileName);
bool	zmap_ExportMap(ZMAP hMap, const char* lpFileName, ZFORMAT hFormat, long nx, long ny);
bool	zmap_ExportMosaic(ZMAP hMap, const char* lpFileName, ZFORMAT hFormat, long nx, long ny, long TileSize, const char* lpProjMapName);
void*	zmap_GetDataPtr(ZMAP hMap);
void*	zmap_GetScanlinePtr(ZMAP hMap, long j);
void**	zmap_GetScanlineArray(ZMAP hMap);
bool	zmap_IsContiguous(ZMAP hMap);
bool	zmap_MakeContiguous(ZMAP hMap);
bool	zmap_LinInterp(ZMAP hMap, double dx, double dy, void* pValue);
void*	zmap_GetMosaicTile(ZMAP hMap, long tx, long ty);
bool	zmap_GenMipmaps(ZMAP hMap, long ResStep, long MaxLevel, long TileSize);
bool	zmap_ClearMipmaps(ZMAP hMap);
ZMAP	zmap_GetMipmapLevel(ZMAP hMap, long MipLevel);
long	zmap_GetMipmapResStep(ZMAP hMap);
long	zmap_GetMipmapMaxLevel(ZMAP hMap);
long	zmap_nMipmaps(ZMAP hMap);
bool	zmap_AllocMipmaps(ZMAP hMap, long MipIncr, long MaxMip);
bool	zmap_GetMipmapLevel2(ZMAP hMap, long MipID, long &MipRes, ZMAP* ppMipMap);

// mosaic tile functions

bool	zmap_tile_GetMinMaxAlt(void* hTile, float& minval, float& maxval);
bool	zmap_tile_GetPixel(void* hTile, long x, long y, void* pValue);
bool	zmap_tile_SetPixel(void* hTile, long x, long y, void* pValue);
bool	zmap_tile_GetTileCoords(void* hTile, long& x, long& y);
bool	zmap_tile_SetLock(void* hTile, bool LockState);
void*	zmap_tile_GetDataPtr(void* hTile);
void*	zmap_tile_GetScanlinePtr(void* hTile, long j);
void**	zmap_tile_GetScanlineArray(void* hTile);
bool	zmap_tile_IsContiguous(void* hTile);
bool	zmap_tile_MakeContiguous(void* hTile);
bool 	zmap_tile_MarkChanged(void* hTile);

//format functions

ZFORMAT zformat_Create(const char* lpMapName, const char* lpLongName, const char* lpFileExt, const char* lpNativeProgramName);
ZFORMAT zformat_CreateGeneric(long MapTypeID, const char* lpLongName, const char* lpFileExt, const char* lpNativeProgramName);
bool	zformat_SetFlags(ZFORMAT hFormat, unsigned long Flags);
unsigned long	zformat_GetFlags(ZFORMAT hFormat);
bool	zformat_SetInfoStr(ZFORMAT hFormat, const char* lpInfoStr);
const char*	zformat_GetExt(ZFORMAT hFormat);
ZLIST	zformat_GetOptionList(ZFORMAT hFormat);
bool	zformat_GetOptionValue(ZFORMAT hFormat, const char* lpOptionName, long VarID, void* pValue);
bool	zformat_SetOptionValue(ZFORMAT hFormat, const char* lpOptionName, long VarID, void* pValue);
ZFORMAT	zformat_GetFormatByExt(const char* lpMapName, long MapTypeID, const char* lpFileExt);
ZFORMAT zformat_GetFormatByExt2(ZMAP hMap, const char* lpFileExt);
ZLIST	zformat_GetFormatList(const char* lpMapName, long MapTypeID);
ZLIST	zformat_GetFormatList2(ZMAP hMap);
ZFORMAT zformat_GetActiveFormat(const char* lpMapName, long MapTypeID);
ZFORMAT zformat_GetDefaultFormat(const char* lpMapName, long MapTypeID);

// prog box functions

bool zprogbox_ShowWnd(ZVAR hProgWnd, unsigned long Flags);
bool zprogbox_HideWnd(ZVAR hProgWnd);
bool zprogbox_SetTitle(ZVAR hProgWnd, const char* lpTitleText);
bool zprogbox_SetProgress(ZVAR hProgWnd, __int64 p, __int64 pmax);

// file selector functions

bool	zfilesel_Init(ZVAR hFileSel, bool OpenFileFlag, const char* lpFileName, const char* lpFilterStr, const char* lpDefaultExt, const char* lpDefaultDir);
bool	zfilesel_GetPathA(ZVAR hFileSel, ZSTR hReturnStr);

// dir selector functions

bool	zdirsel_Init(ZVAR hDirSel, const char* lpStartDir);
bool	zdirsel_GetPathA(ZVAR hDirSel, ZSTR hReturnStr);

// combo box functions

bool	zcombosel_Init(ZVAR hComboSel, const char* lpOptionsStr, const char* lpCurSelStr, unsigned long Flags);
long	zcombosel_nOptions(ZVAR hComboSel);
bool	zcombosel_GetOptionsA(ZVAR hComboSel, ZSTR hReturnStr);
bool	zcombosel_SetOptionsA(ZVAR hComboSel, const char* lpOptionsStr);
long	zcombosel_AddOptionA(ZVAR hComboSel, const char* lpOptionStr);
bool	zcombosel_GetOptionText(ZVAR hComboSel, long index, ZSTR hReturnStr);
long	zcombosel_FindOptionA(ZVAR hComboSel, const char* lpOptionStr);
long	zcombosel_GetCurSelI(ZVAR hComboSel);
bool	zcombosel_SetCurSelI(ZVAR hComboSel, long CurSelID);
bool	zcombosel_GetCurSelA(ZVAR hComboSel, ZSTR hReturnStr);
bool	zcombosel_SetCurSelA(ZVAR hComboSel, const char* lpCurSelStr);
bool	zcombosel_SetSelI(ZVAR hComboSel, long index, bool State);
bool	zcombosel_SetSelA(ZVAR hComboSel, const char* lpOptionStr, bool State);
bool	zcombosel_GetSelI(ZVAR hComboSel, long index);
bool	zcombosel_GetSelA(ZVAR hComboSel, const char* lpOptionStr);
unsigned long zcombosel_GetFlags(ZVAR hComboSel);

// buffer function handle defs

bool	zbuffer_Init(ZVAR hBuf, long nItems, long ItemSize);
bool	zbuffer_InitByType(ZVAR hBuf, long nItems, long ItemTypeID);
bool	zbuffer_Free(ZVAR hBuf);
long	zbuffer_nItems(ZVAR hBuf);
long	zbuffer_MemSize(ZVAR hBuf);
long	zbuffer_ItemSize(ZVAR hBuf);
long	zbuffer_ItemType(ZVAR hBuf);
void*	zbuffer_GetPtr(ZVAR hBuf);
bool	zbuffer_GetValue(ZVAR hBuf, long index, void* pValue);
bool	zbuffer_SetValue(ZVAR hBuf, long index, void* pValue);

// extension functions

ZFUNC	zfunc_Load(const char* lpFnName, const char* lpExportName, long rvalVarID, ZLIST hArgList);
ZFUNC	zfunc_GetFunc(const char* lpFnName);
long	zfunc_GetReturnTypeID(ZFUNC hFunc);
bool	zfunc_GetArgListPrototype(ZFUNC hFunc, ZLIST hArgList);
bool	zfunc_Execute(ZFUNC hFunc, ZLIST hArgList, ZVAR* ppRval);

// script functions
bool	zscript_Execute(const char* lpLangName, const char* lpScript, ZVAR hRval = 0);

