#include "ZlibWrap.h"
#include <zeolite.h>
#include <zfunc.h>
#include <zvar.h>
#include <zlist.h>
#include <zbuffer.h>

namespace zlib {


static CzFunc* lpCompress = NULL;
static CzFunc* lpCompressBound = NULL;
static CzFunc* lpUncompress = NULL;

bool Init() {

	lpCompress      = new CzFunc("zlib.Compress");
	lpCompressBound = new CzFunc("zlib.CompressBound");
	lpUncompress    = new CzFunc("zlib.Uncompress");
	
	return true;
}
void Free() {
	if(lpCompress) {
		delete lpCompress;
		lpCompress = NULL;
	}
	if(lpCompressBound) {
		delete lpCompressBound;
		lpCompressBound = NULL;
	}
	if(lpUncompress) {
		delete lpUncompress;
		lpUncompress = NULL;
	}
}

long Compress(void* pDest, unsigned long* pDestLen, void* pSrc, unsigned long SrcLen) {
	
	CzFunc* pf = lpCompress;
	if(!pf) return -1;

	CzVar DestLen(VarID_uint);
	DestLen.SetValue(VarID_uint, pDestLen);
	ZVAR hDestLen = DestLen;
	
	long argid = 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_voidptr, &pDest))	return 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_hvar, &hDestLen))	return 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_voidptr, &pSrc))	return 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_uint, &SrcLen))		return 0;

	if(!pf->Execute2()) {
		return -1;
	}

	DestLen.GetValue(VarID_uint, pDestLen); // recover dest len value
	
	long rval = 0;
	if(!pf->m_rval.GetValue(VarID_int, &rval)) {
		return -1;
	}
	
	return rval;
}

unsigned long CompressBound(unsigned long SrcLen) {
	
	CzFunc* pf = lpCompressBound;
	if(!pf) return -1;

	long argid = 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_uint, &SrcLen))		return 0;

	if(!pf->Execute2()) {
		return -1;
	}

	unsigned long rval = 0;
	if(!pf->m_rval.GetValue(VarID_uint, &rval)) {
		return -1;
	}
	
	return rval;
}

long Uncompress(void* pDest, unsigned long* pDestLen, void* pSrc, unsigned long SrcLen) {

	CzFunc* pf = lpUncompress;
	if(!pf) return -1;

	CzVar DestLen(VarID_uint);
	DestLen.SetValue(VarID_uint, pDestLen);
	ZVAR hDestLen = DestLen;
	
	long argid = 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_voidptr, &pDest))	return 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_hvar, &hDestLen))	return 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_voidptr, &pSrc))	return 0;
	if(!pf->m_args.SetItemValueI(argid++, VarID_uint, &SrcLen))		return 0;

	if(!pf->Execute2()) {
		return -1;
	}

	DestLen.GetValue(VarID_uint, pDestLen); // recover dest len value
	
	long rval = 0;
	if(!pf->m_rval.GetValue(VarID_int, &rval)) {
		return -1;
	}
	
	return rval;

}

}