#include <Zeolite.h>
#include "atMeshDecimator.h"
#include <zClasses.h>
#include <zFunc.h>

bool atMeshDecimator_DecimateHF(ZMAP pHF, CzMesh* pMesh, float MaxError, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("atMeshDecimator_DecimateHF error:\r\n - invalid mesh handle");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	if(!hMesh) {
		if(!pMesh->Create()) {
			zeoReportError("atMeshDecimator_DecimateHF error:\r\n - cannot create mesh");
			return false;
		}
		
		hMesh = pMesh->GetZVAR();
	}

	if(MAP_Heightfield!=zmap_GetMapType(pHF)) {
		zeoReportError("atMeshDecimator_DecimateHF error:\r\n - heightfield is not initialised");
		return false;
	}

	CzFunc fDeci;
	if(!fDeci.GetFunc("calc.mesh.DecimateHF")) {
		zeoReportError("atMeshDecimator_DecimateHF error:\r\n - cannot get function handle");
		return false;
	}

	fDeci.m_args.SetItemValueI(0, VarID_hvar, &pHF);
	fDeci.m_args.SetItemValueI(1, VarID_hvar, &hMesh);
	fDeci.m_args.SetItemValueI(2, VarID_float, &MaxError);
	fDeci.m_args.SetItemValueI(3, VarID_uint, &Flags);
	fDeci.m_args.SetItemValueI(4, VarID_hvar, &hOptions);

	if(!fDeci.Execute2()) {
		zeoWriteToLog("atMeshDecimator_DecimateHF error:\r\n - call to extension function failed");
		return false;
	}

	bool rval = false;
	if(!fDeci.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("atMeshDecimator_DecimateHF error:\r\n - cannot get function rval");
		return false;
	}
	if(!rval) {
		zeoWriteToLog("atMeshDecimator_DecimateHF error:\r\n - call to extension function returned false");
		return false;
	}

	if(!pMesh->LoadHandles()) {
		zeoWriteToLog("atMeshDecimator_DecimateHF error:\r\n - cannot load mesh handles (invalid mesh)");
		return false;
	}
	
	return true;
}
bool atMeshDecimator_DecimateHF_UI(ZMAP pHF, CzMesh* pMesh, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("atMeshDecimator_DecimateHF_UI error:\r\n - invalid mesh handle");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	if(!hMesh) {
		if(!pMesh->Create()) {
			zeoReportError("atMeshDecimator_DecimateHF_UI error:\r\n - cannot create mesh");
			return false;
		}
		
		hMesh = pMesh->GetZVAR();
	}

	if(MAP_Heightfield!=zmap_GetMapType(pHF)) {
		zeoReportError("atMeshDecimator_DecimateHF_UI error:\r\n - heightfield is not initialised");
		return false;
	}

	CzFunc fDeci;
	if(!fDeci.GetFunc("UI.MeshDecimator.DecimateUI")) {
		zeoReportError("atMeshDecimator_DecimateHF_UI error:\r\n - cannot get function handle");
		return false;
	}

	fDeci.m_args.SetItemValueI(0, VarID_hvar, &pHF);
	fDeci.m_args.SetItemValueI(1, VarID_hvar, &hMesh);
	fDeci.m_args.SetItemValueI(2, VarID_uint, &Flags);
	fDeci.m_args.SetItemValueI(3, VarID_hvar, &hOptions);

	if(!fDeci.Execute2()) {
		zeoWriteToLog("atMeshDecimator_DecimateHF_UI error:\r\n - call to extension function failed");
		return false;
	}

	bool rval = false;
	if(!fDeci.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("atMeshDecimator_DecimateHF_UI error:\r\n - cannot get function rval");
		return false;
	}
	if(!rval) {
		zeoWriteToLog("atMeshDecimator_DecimateHF_UI error:\r\n - call to extension function returned false");
		return false;
	}

	if(!pMesh->LoadHandles()) {
		zeoWriteToLog("atMeshDecimator_DecimateHF_UI error:\r\n - cannot load mesh handles (invalid mesh)");
		return false;
	}
	
	return true;
}