#include <stdafx.h>
#include <Zeolite.h>
#include "zbin.h"

#include <zFunc.h>

static CzFunc* pzbOpen = NULL;
static CzFunc* pzbClose = NULL;
static CzFunc* pzbSeek = NULL;
static CzFunc* pzbRewind = NULL;
static CzFunc* pzbTell = NULL;
static CzFunc* pzbRead = NULL;
static CzFunc* pzbReadVar = NULL;
static CzFunc* pzbReadVar2 = NULL;
static CzFunc* pzbReadVarData = NULL;
static CzFunc* pzbWrite = NULL;
static CzFunc* pzbWriteVar = NULL;
static CzFunc* pzbWriteVarData = NULL;
static CzFunc* pzbCodec = NULL;
static CzFunc* pzbRwMode = NULL;
static CzFunc* pzbReadHeader = NULL;
static CzFunc* pzbWriteHeader = NULL;


namespace zbin {
void Init() { // gets function handles

	pzbOpen = new CzFunc("file.ZBIN.zbOpen");
	pzbClose = new CzFunc("file.ZBIN.zbClose");
	pzbSeek = new CzFunc("file.ZBIN.zbSeek");
	pzbRewind = new CzFunc("file.ZBIN.zbRewind");
	pzbTell = new CzFunc("file.ZBIN.zbTell");
	pzbRead = new CzFunc("file.ZBIN.zbRead");
	pzbReadVar = new CzFunc("file.ZBIN.zbReadVar");
	pzbReadVar2 = new CzFunc("file.ZBIN.zbReadVar2");
	pzbReadVarData = new CzFunc("file.ZBIN.zbReadVarData");
	pzbWrite = new CzFunc("file.ZBIN.zbWrite");
	pzbWriteVar = new CzFunc("file.ZBIN.zbWriteVar");
	pzbWriteVarData = new CzFunc("file.ZBIN.zbWriteVarData");
	pzbCodec = new CzFunc("file.ZBIN.zbCodec");
	pzbRwMode = new CzFunc("file.ZBIN.zbRwMode");
	pzbReadHeader = new CzFunc("file.ZBIN.zbReadHeader");
	pzbWriteHeader = new CzFunc("file.ZBIN.zbWriteHeader");
}
void Free() { // releases function handles
	delete pzbOpen;
	delete pzbClose;
	delete pzbSeek;
	delete pzbRewind;
	delete pzbTell;
	delete pzbRead;
	delete pzbReadVar;
	delete pzbReadVar2;
	delete pzbReadVarData;
	delete pzbWrite;
	delete pzbWriteVar;
	delete pzbWriteVarData;
	delete pzbCodec;
	delete pzbRwMode;

	pzbOpen = NULL;
	pzbClose = NULL;
	pzbSeek = NULL;
	pzbRewind = NULL;
	pzbTell = NULL;
	pzbRead = NULL;
	pzbReadVar = NULL;
	pzbReadVar2 = NULL;
	pzbReadVarData = NULL;
	pzbWrite = NULL;
	pzbWriteVar = NULL;
	pzbWriteVarData = NULL;
	pzbCodec = NULL;
	pzbRwMode = NULL;
}

//
// raw binary files
//

long Load(const char* lpFileName, ZLIST hContainerList) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.Load")) {
		zeoReportError("zbin_Load error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_Load error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hContainerList)) {
		zeoReportError("zbin_Load error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_Load error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_Load error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
	
}
long Save(const char* lpFileName, ZVAR hVar) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.Save")) {
		zeoReportError("zbin_Save error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_Save error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hVar)) {
		zeoReportError("zbin_Save error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_Save error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_Save error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}

//
// gzip files
//

long LoadGzip(const char* lpFileName, ZLIST hContainerList) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.LoadGzip")) {
		zeoReportError("zbin_LoadGzip error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_LoadGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hContainerList)) {
		zeoReportError("zbin_LoadGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_LoadGzip error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_LoadGzip error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
	
}
long SaveGzip(const char* lpFileName, ZVAR hVar) {

	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.SaveGzip")) {
		zeoReportError("zbin_SaveGzip error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!zstr_SetText(fBufferWrite.GetArgI(0), lpFileName)) {
		zeoReportError("zbin_SaveGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hVar)) {
		zeoReportError("zbin_SaveGzip error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_SaveGzip error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_SaveGzip error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}

//
// raw buffers
//

long LoadBuf(ZBUF hBuffer, ZLIST hContainerList) {
											
	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.LoadBuf")) {
		zeoReportError("zbin_LoadBuf error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(0, VarID_hvar, &hBuffer)) {
		zeoReportError("zbin_LoadBuf error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hContainerList)) {
		zeoReportError("zbin_LoadBuf error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_LoadBuf error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_LoadBuf error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}


long SaveBuf(ZBUF hBuffer, ZVAR hVar) {
											
	CzFunc fBufferWrite;
	if(!fBufferWrite.GetFunc("file.ZBIN.SaveBuf")) {
		zeoReportError("zbin_SaveBuf error:\r\n - cannot get ZBIN function");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(0, VarID_hvar, &hBuffer)) {
		zeoReportError("zbin_SaveBuf error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.SetArgValueI(1, VarID_hvar, &hVar)) {
		zeoReportError("zbin_SaveBuf error:\r\n - cannot set ZBIN function arg");
		return -1;
	}
	
	if(!fBufferWrite.Execute2()) {
		zeoReportError("zbin_SaveBuf error:\r\n - call to ZBIN function failed");
		return -1;
	}
	
	long rval = -1;
	if(!fBufferWrite.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("zbin_SaveBuf error:\r\n - cannot get rval");
		return -1;
	}
	
	return rval;
}

}


//
// low-level functions
//

long zbOpen(ZBINFILE* fp, const char* lpFileName, long RwMode) {
	CzFunc* pfn = pzbOpen;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;
	
	// create container (deleted in zbClose)
	ZVAR hVar = zvar_Create(VarID_voidptr);

	long argid = 0;

	if(!pfn->m_args.SetItemValueI(argid++, VarID_hvar, &hVar)) {
		zvar_Delete(hVar);
		return ZBIN_ERROR_FAILED;
	}
	if(!zstr_SetText(pfn->m_args.GetItemI(argid++), lpFileName)) {
		zvar_Delete(hVar);
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->m_args.SetItemValueI(argid++, VarID_int, &RwMode)) {
		zvar_Delete(hVar);
		return ZBIN_ERROR_FAILED;
	}

	
	if(!pfn->Execute2()) {
		zvar_Delete(hVar);
		return ZBIN_ERROR_FAILED;
	}

	*fp = hVar;

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbClose(ZBINFILE fp) {
	CzFunc* pfn = pzbClose;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	// delete container (created in zbOpen)
	zvar_Delete(fp);

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbSeek(ZBINFILE fp, __int64 Offset, long Origin) {
	CzFunc* pfn = pzbSeek;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_int64, &Offset)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_int, &Origin)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbRewind(ZBINFILE fp) {
	CzFunc* pfn = pzbRewind;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

__int64 zbTell(ZBINFILE fp) {
	CzFunc* pfn = pzbTell;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	__int64 rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int64, &rval);
	return rval;
}

long zbRead(ZBINFILE fp, void* pData, long nBytes) {
	CzFunc* pfn = pzbRead;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_voidptr, &pData)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_int, &nBytes)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbReadVar2(ZBINFILE fp, ZLIST hParent) {
	CzFunc* pfn = pzbReadVar;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	
	if(!pfn->SetArgValueI(argid++, VarID_hvar, &hParent)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}
long zbReadVar2(ZBINFILE fp, ZVAR* phVar) {
	CzFunc* pfn = pzbReadVar2;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	
	if(!pfn->SetArgValueI(argid++, VarID_varref, &phVar)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbReadVarData(ZBINFILE fp, ZVAR hVar) {
	CzFunc* pfn = pzbReadVarData;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_hvar, &hVar)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbWrite(ZBINFILE fp, void* pData, long nBytes) {
	CzFunc* pfn = pzbWrite;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_voidptr, &pData)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_int, &nBytes)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbWriteVar(ZBINFILE fp, ZVAR hVar) {
	CzFunc* pfn = pzbWriteVar;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_hvar, &hVar)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbWriteVarData(ZBINFILE fp, ZVAR hVar) {
	CzFunc* pfn = pzbWriteVarData;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}
	if(!pfn->SetArgValueI(argid++, VarID_hvar, &hVar)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	long rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbCodec(ZBINFILE fp) {
	CzFunc* pfn = pzbCodec;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	int rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbRwMode(ZBINFILE fp) {
	CzFunc* pfn = pzbRwMode;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	int rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}

long zbWriteHeader(ZBINFILE fp) {
	CzFunc* pfn = pzbWriteHeader;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	int rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}
long zbReadHeader(ZBINFILE fp) {
	CzFunc* pfn = pzbReadHeader;

	if(!pfn)			return ZBIN_ERROR_FAILED;
	if(!pfn->GetZVAR()) return ZBIN_ERROR_NOT_SUPPORTED;

	int argid = 0;
	if(!zvar_CopyValue(pfn->GetArgI(argid++), fp)) {
		return ZBIN_ERROR_FAILED;
	}

	if(!pfn->Execute2()) {
		return ZBIN_ERROR_FAILED;
	}

	int rval = ZBIN_ERROR_FAILED;
	pfn->m_rval.GetValue(VarID_int, &rval);
	return rval;
}