#ifndef __ZBIN_INCLUDED__
#define __ZBIN_INCLUDED__

#include <Zeolite.h>

// codecs
#define ZBIN_CODEC_NONE		0	// not initialised
#define ZBIN_CODEC_RAW		1	// uses read/write in binary mode
#define ZBIN_CODEC_BUFFER	2	// writes to a memory mapped buffer
#define ZBIN_CODEC_GZIP		3	// uses read/write with gzip

#define ZBIN_RWMODE_NONE		0
#define ZBIN_RWMODE_READ		1
#define ZBIN_RWMODE_WRITE		2

#define ZBIN_ERROR_SUCCESS			0
#define ZBIN_ERROR_FAILED			-1
#define ZBIN_ERROR_INVALID_PARAM	-2
#define ZBIN_ERROR_INVALID_CODEC	-3
#define ZBIN_ERROR_INVALID_RWMODE	-4
#define ZBIN_ERROR_OPEN_FAILED		-5
#define ZBIN_ERROR_READ_FAILED		-6
#define ZBIN_ERROR_WRITE_FAILED		-7
#define ZBIN_ERROR_SEEK_FAILED		-8
#define ZBIN_ERROR_FILE_OPEN		-9
#define ZBIN_ERROR_FILE_CLOSED		-10
#define ZBIN_ERROR_OUT_OF_RANGE		-11
#define ZBIN_ERROR_NOT_SUPPORTED	-12

#define ZBINFILE ZVAR

// high-level functions
namespace zbin {

	void Init(); // gets function handles
	void Free(); // releases function handles

	long Load(const char* lpFileName, ZLIST hParent);
	long LoadGzip(const char* lpFileName, ZLIST hParent);
	long LoadBuf(ZBUF hBuffer, ZLIST hParent);
	long Save(const char* lpFileName, ZVAR hVar);
	long SaveGzip(const char* lpFileName, ZVAR hVar);
	long SaveBuf(ZBUF hBuffer, ZVAR hVar);
}

// low level functions

long zbOpen(ZBINFILE* fp, const char* lpFileName, long RwMode);
long zbClose(ZBINFILE fp);
long zbSeek(ZBINFILE fp, __int64 Offset, long Origin = SEEK_SET);
long zbRewind(ZBINFILE fp);
__int64 zbTell(ZBINFILE fp);
long zbRead(ZBINFILE fp, void* pData, long nBytes);
long zbReadVar(ZBINFILE fp, ZLIST hParent);
long zbReadVar2(ZBINFILE fp, ZVAR* phVar);
long zbReadVarData(ZBINFILE fp, ZVAR hVar);
long zbWrite(ZBINFILE fp, void* pData, long nBytes);
long zbWriteVar(ZBINFILE fp, ZVAR hVar);
long zbWriteVarData(ZBINFILE fp, ZVAR hVar);
long zbCodec(ZBINFILE fp);
long zbRwMode(ZBINFILE fp);
long zbWriteHeader(ZBINFILE fp);
long zbReadHeader(ZBINFILE fp);

#endif