#include "Zeolite.h"
#include "zclimate.h"
#include "zfunc.h"
#include "zstr.h"

CzClimate::CzClimate(void) {
	m_hTypeList = NULL;
}

CzClimate::~CzClimate(void) {
}

bool CzClimate::Attach(ZVAR hVar) {

	if(VarID_climate!=zvar_GetTypeID(hVar)) {
		zeoReportError("CzClimate::Attach error:\r\n - no climate var attached");
		return false;
	}

	//
	// now get land type list
	//
	CzFunc f;
	if(!f.GetFunc("climate.GetTypeList")) {
		zeoReportError("CzClimate::Attach error:\r\n - cannot get func handle");
		return false;
	}
	if(!f.SetArgValueI(0, VarID_hvar, &hVar)) {
		zeoReportError("CzClimate::Attach error:\r\n - cannot set arg value");
		return false;
	}
	if(!f.Execute2()) {
		zeoReportError("CzClimate::Attach error:\r\n - call to func failed");
		return false;
	}
	m_hTypeList = NULL;
	if(!f.m_rval.GetValue(VarID_hvar, &m_hTypeList)) {
		zeoReportError("CzClimate::Attach error:\r\n - cannot get return value");
		return false;
	}

	// now finally attach to var
	if(!CzVar::Attach(hVar)) {
		zeoReportError("CzClimate::Attach error:\r\n - call to CzVar::Attach failed");
		return false;
	}

	return true;
}

long CzClimate::nTypes() {	return zlist_nItems(m_hTypeList); }

ZVAR CzClimate::GetType(long index) {
	return zlist_GetItemI(m_hTypeList, index);
}

long CzClimate::FindTypeA(const char* lpTypeName) {

	if(!m_hVar) {
		zeoReportError("CzClimate::FindTypeA error:\r\n - no climate var attached");
		return -1;
	}

	CzFunc f;
	if(!f.GetFunc("climate.FindTypeA")) {
		zeoReportError("CzClimate::FindTypeA error:\r\n - cannot get func handle");
		return -1;
	}
	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		zeoReportError("CzClimate::FindTypeA error:\r\n - cannot set arg value");
		return -1;
	}
	if(!zstr_SetText(f.GetArgI(1), lpTypeName)) {
		zeoReportError("CzClimate::FindTypeA error:\r\n - cannot set arg value (2)");
		return -1;
	}
	if(!f.Execute2()) {
		zeoReportError("CzClimate::FindTypeA error:\r\n - call to func fauled");
		return -1;
	}
	int rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("CzClimate::FindTypeA error:\r\n - cannot get rval");
		return -1;
	}

	return rval;
}

bool CzClimate::GetClimateName(ZSTR hNameVar) {
	
	if(!m_hVar) {
		zeoReportError("CzClimate::GetClimateName error:\r\n - no climate var attached");
		return false;
	}

	if(!zvar_IsType(hNameVar, VarID_string)) {
		zeoReportError("CzClimate::GetClimateName error:\r\n - argument is invalid");
		return false;
	}

	CzFunc f;
	if(!f.GetFunc("climate.GetName")) {
		zeoReportError("CzClimate::GetClimateName error:\r\n - cannot get func handle");
		return false;
	}
	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		zeoReportError("CzClimate::GetClimateName error:\r\n - cannot set arg value");
		return false;
	}
	if(!f.Execute2()) {
		zeoReportError("CzClimate::GetClimateName error:\r\n - call to func fauled");
		return false;
	}
	if(!zvar_CopyValue(hNameVar, (ZVAR)f.m_rval)) {
		zeoReportError("CzClimate::GetClimateName error:\r\n - cannot copy rval");
		return false;
	}

	return true;
}

long CzClimate::AddToProject() {
	if(!zvar_IsType(m_hVar, VarID_climate)) {
		zeoReportError("CzClimate::AddToProject error:\r\n - no climate var attached");
		return -1;
	}
	
	CzFunc f;
	if(!f.GetFunc("project.AddClimateToProject")) {
		zeoReportError("CzClimate::AddToProject error:\r\n - cannot get func handle");
		return -1;
	}

	CzStr CliName;
	if(!GetClimateName((ZVAR)CliName)) {
		zeoReportError("CzClimate::AddToProject error:\r\n - cannot get climate name");
		return -1;
	}

	if(!zvar_CopyValue(f.GetArgI(0), (ZVAR)CliName)) {
		zeoReportError("CzClimate::AddToProject error:\r\n - cannot set arg value");
		return -1;
	}
	if(!f.Execute2()) {
		zeoReportError("CzClimate::AddToProject error:\r\n - call to func failed");
		return false;
	}

	long rval;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("CzClimate::AddToProject error:\r\n - cannot get return value");
		return -1;
	}

	return rval;
}

// getting climate IDs in application and project
long CzClimate::GetProjectID() {
	
	if(!zvar_IsType(m_hVar, VarID_climate)) {
		zeoReportError("CzClimate::GetProjectID error:\r\n - no climate var attached");
		return -1;
	}

	CzFunc f;
	if(!f.GetFunc("climate.GetProjID")) {
		zeoReportError("CzClimate::GetProjectID error:\r\n - cannot get func handle");
		return -1;
	}

	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		zeoReportError("CzClimate::GetProjectID error:\r\n - cannot set arg value");
		return -1;
	}
	if(!f.Execute2()) {
		zeoReportError("CzClimate::GetProjectID error:\r\n - call to func fauled");
		return -1;
	}

	long rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("CzClimate::GetProjectID error:\r\n - cannot get rval");
		return -1;
	}

	return rval;
}
long CzClimate::GetApplicationID() {
	if(!zvar_IsType(m_hVar, VarID_climate)) {
		zeoReportError("CzClimate::GetApplicationID error:\r\n - no climate var attached");
		return -1;
	}

	CzFunc f;
	if(!f.GetFunc("climate.GetAppID")) {
		zeoReportError("CzClimate::GetApplicationID error:\r\n - cannot get func handle");
		return -1;
	}

	if(!f.SetArgValueI(0, VarID_hvar, &m_hVar)) {
		zeoReportError("CzClimate::GetApplicationID error:\r\n - cannot set arg value");
		return -1;
	}
	if(!f.Execute2()) {
		zeoReportError("CzClimate::GetApplicationID error:\r\n - call to func fauled");
		return -1;
	}

	long rval = -1;
	if(!f.m_rval.GetValue(VarID_int, &rval)) {
		zeoReportError("CzClimate::GetApplicationID error:\r\n - cannot get rval");
		return -1;
	}

	return rval;
}