#include "Zeolite.h"
#include "zfunc.h"

CzFunc::CzFunc(void) {
}
CzFunc::CzFunc(const char* lpFuncName) {
	GetFunc(lpFuncName);
}

CzFunc::~CzFunc(void) {
}

bool CzFunc::Load(const char* lpFnName, const char* lpExportName, long rvalVarID, ZLIST hArgList) {
	return Attach(zfunc_Load(lpFnName, lpExportName, rvalVarID, hArgList));
}
bool CzFunc::GetFunc(const char* lpFnName) {
	return Attach(zfunc_GetFunc(lpFnName));
}
long CzFunc::GetReturnTypeID() {
	if(!m_hVar)	return 0;

	return zfunc_GetReturnTypeID(m_hVar);
}
bool CzFunc::GetArgListPrototype(ZLIST hArgList) {
	if(!m_hVar)	return false;

	return zfunc_GetArgListPrototype(m_hVar, hArgList);
}
ZVAR CzFunc::Execute() {
	if(!m_hVar)	return NULL;

	ZVAR hArgList = m_args.GetZVAR();
	ZVAR hRval = m_rval.GetZVAR();

	if(!zfunc_Execute(m_hVar, hArgList, &hRval)) {
		return NULL;
	}
	
	return hRval;
}
bool CzFunc::Execute2(ZLIST hArgList, ZVAR* ppRval) {

	if(!m_hVar)	{
		return false;
	}

	bool rval = false;

	if(!hArgList) {
		hArgList = m_args.GetZVAR();
	}

	bool hRval_provided = false;
	ZVAR hRval = m_rval.GetZVAR(); // default rval is from function
	if(ppRval) {
		if(*ppRval) {
			// if you got here, the user provided a handle to a return value variable
			// (in this case, we must copy the rval back to the CzFunc::m_rval variable after execution)
			hRval = *ppRval;
			hRval_provided = true;
		}
	}

	if(!zfunc_Execute(m_hVar, hArgList, &hRval)) {
		return false;
	}

	if(hRval_provided) {
		// if an rval was provided, we must copy back now to the CzFunc::m_rval local instance
		long VarID = zvar_GetTypeID(hRval);
		if(VarID!=m_rval.GetTypeID()) {
			if(!m_rval.ReInit(VarID)) {
				zeoReportError("CzFunc::Execute2 error:\r\n - cannot re-initialise local rval");
				return false;
			}
		}
		if(!m_rval.CopyValue(hRval)) {
			zeoReportError("CzFunc::Execute2 error:\r\n - cannot copy rval back to local instance");
			return false;
		}
	} else {
		// user provided ppRval was handle to NULL hVar, so we return a pointer to the user
		if(ppRval) {
			*ppRval = hRval;
		}
	}
	
	return true;
}

bool CzFunc::Attach(ZVAR hVar) {
	if(!CzVar::Attach(hVar))	return false;

	if(!IsType(VarID_zeofunc)) {
		if(!Delete()) {
			zeoReportError("CzFunc::Attach error:\r\n - call to Delete() failed (1)");
		}
		return false;
	}

	// get the arg list
	if(!GetArgListPrototype(m_args.GetZVAR())) {
		if(!Delete()) {
			zeoReportError("CzFunc::Attach error:\r\n - call to Delete() failed (2)");
		}
		return false;
	}

	// (re)initialise the return value
	long VarID = zfunc_GetReturnTypeID(hVar);
	if(m_rval.GetTypeID()==VarID_INVALID) {
		if(!m_rval.Create(zfunc_GetReturnTypeID(hVar))) {
			zeoReportError("CzFunc::Attach error:\r\n - cannot create rval");
			return false;
		}
	} else
	if(m_rval.GetTypeID()!=VarID) {
		if(!m_rval.ReInit(VarID)) {
			zeoReportError("CzFunc::Attach error:\r\n - cannot reinit rval");
			return false;
		}
	}

	return true;
}

ZVAR CzFunc::GetArgI(long index) {
	if(!m_hVar)	
		return false;

	return m_args.GetItemI(index);
}
ZVAR CzFunc::GetArgA(const char* lpArgName) {
	if(!m_hVar)	
		return false;

	return m_args.GetItemA(lpArgName);
}
bool CzFunc::SetArgValueI(long index, long VarID, void* pValue) {
	if(!m_hVar)	
		return false;

	return m_args.SetItemValueI(index, VarID, pValue);
}
bool CzFunc::SetArgValueA(const char* lpArgName, long VarID, void* pValue) {
	if(!m_hVar)	
		return false;

	return m_args.SetItemValueA(lpArgName, VarID, pValue);
}