#include "Zeolite.h"
#include "zlist.h"

CzList::CzList() {
	Create();
}
CzList::CzList(ZLIST hList) {
	Attach(hList);
}

CzList::~CzList(void) {
}

bool CzList::Create() {
	return CzVar::Create(VarID_varlist);
}
bool CzList::CreateShared(const char* lpVarName) {
	return CzVar::CreateShared(VarID_varlist, lpVarName);
}

long CzList::nItems() {
	if(!m_hVar)	return 0;

	return zlist_nItems(m_hVar);
}
ZVAR CzList::CreateItem(long VarID, const char* lpVarName) {
	if(!m_hVar)	return 0;

	return zlist_CreateItem(m_hVar, VarID, lpVarName);
}
ZVAR CzList::GetOrCreateItem(long VarID, const char* lpVarName) {
	if(!m_hVar)	return 0;

	ZVAR TempVar = 0;

	if(TempVar = GetItem(lpVarName)) {
		// var exists, now check type
		if(zvar_IsType(TempVar, VarID))
				return TempVar;
		else	return false;
	}
	
	return CreateItem(VarID, lpVarName);
}
ZVAR CzList::GetItem(long index) {
	return GetItemI(index);
}
ZVAR CzList::GetItem(const char* lpVarName) {
	return GetItemA(lpVarName);
}
ZVAR CzList::GetItemI(long index) {
	if(!m_hVar)	return 0;

	return zlist_GetItemI(m_hVar, index);
}
ZVAR CzList::GetItemA(const char* lpVarName) {
	if(!m_hVar)	return 0;

	return zlist_GetItemA(m_hVar, lpVarName);
}

bool CzList::DelItem(long index) {
	return DelItemI(index);
}
bool CzList::DelItem(const char* lpVarName) {
	return DelItemA(lpVarName);
}
bool CzList::DelItemI(long index) {
	if(!m_hVar)	return false;

	return zlist_DelItemI(m_hVar, index);
}
bool CzList::DelItemA(const char* lpVarName) {
	if(!m_hVar)	return false;

	return zlist_DelItemA(m_hVar, lpVarName);
}
bool CzList::DelAll() {
	if(!m_hVar)	return false;

	return zlist_DelAll(m_hVar);
}

bool CzList::Append(ZLIST hList) {
	if(!m_hVar)	return false;

	return zlist_Append(m_hVar, hList);
}

bool CzList::GetItemValue(const char* lpItemName, long VarID, void* pValue) {
	return GetItemValueA(lpItemName, VarID, pValue);
}
bool CzList::GetItemValue(long index, long VarID, void* pValue) {
	return GetItemValueI(index, VarID, pValue);
}
bool CzList::GetItemValueA(const char* lpItemName, long VarID, void* pValue) {
	if(!m_hVar)	return false;

	ZVAR hVar;
	if(!(hVar = zlist_GetItemA(m_hVar, lpItemName)))
		return false;

	return zvar_GetValueEx(hVar, VarID, pValue);

}
bool CzList::GetItemValueI(long index, long VarID, void* pValue) {
	if(!m_hVar)	return false;

	ZVAR hVar;
	if(!(hVar = zlist_GetItemI(m_hVar, index)))
		return false;

	return zvar_GetValueEx(hVar, VarID, pValue);
}
bool CzList::SetItemValue(const char* lpItemName, long VarID, void* pValue) {
	return SetItemValueA(lpItemName, VarID, pValue);
}
bool CzList::SetItemValue(long index, long VarID, void* pValue) {
	return SetItemValueI(index, VarID, pValue);
}
bool CzList::SetItemValueA(const char* lpItemName, long VarID, void* pValue) {
	if(!m_hVar)	return false;

	ZVAR hVar;
	if(!(hVar = zlist_GetItemA(m_hVar, lpItemName))) {
		if(!(hVar = zlist_CreateItem(m_hVar, VarID, lpItemName)))
			return false;
	}

	return zvar_SetValueEx(hVar, VarID, pValue);
}
bool CzList::SetItemValueI(long index, long VarID, void* pValue) {
	if(!m_hVar)	return false;

	ZVAR hVar;
	if(!(hVar = zlist_GetItemI(m_hVar, index)))
		return false;

	return zvar_SetValueEx(hVar, VarID, pValue);
}

const char* CzList::GetItemName(long index) {
	if(!m_hVar)	return 0;

	ZVAR hVar;
	if(!(hVar = zlist_GetItemI(m_hVar, index)))
		return 0;

	return zvar_GetName(hVar);
}

bool CzList::Sort(long From, long To) {
	if(!m_hVar)	return 0;

	return zlist_SortI(m_hVar, From, To);
}
bool CzList::Sort(const char* lpItemName, long To) {
	if(!m_hVar)	return 0;

	return zlist_SortA(m_hVar, lpItemName, To);
}
long CzList::FindItemA(const char* lpItemName, long StartAt) {
	if(!lpItemName)
		return -1;
	if(!strlen(lpItemName))
		return -1;

	long n = nItems();
	for(long i=StartAt; i<n; i++) {
		if(0==strcmp(GetItemName(i), lpItemName)) {
			return i;
		}
	}

	return -1;
}
long CzList::FindItemH(ZVAR hItem, long StartAt) {
	if(!hItem)
		return -1;

	long n = nItems();
	for(long i=StartAt; i<n; i++) {
		if(hItem==GetItemI(i)) {
			return i;
		}
	}

	return -1;
}

bool CzList::ItemExists(const char* lpItemName) {
	if(GetItemA(lpItemName)==NULL)
		return false;
	
	return true;
}