#include "Zeolite.h"
#include "zmap.h"

CzMap::CzMap(void) {
	Create();

	// direct interface variables
	m_DirectInterfaceFlag = false;
	m_nx = 0;
	m_ny = 0;
	m_nTilesX = 0;
	m_nTilesY = 0;
	m_nPixels = 0;
	m_TileSize = 0;
	m_PixelMemSize = 0;
	m_AllocMode = 0;
	m_pMem = NULL;
	m_Contiguous = false;
}

CzMap::CzMap(ZMAP hMap) {
	Attach(hMap);

	// direct interface variables
	m_DirectInterfaceFlag = false;
	m_nx = 0;
	m_ny = 0;
	m_nTilesX = 0;
	m_nTilesY = 0;
	m_nPixels = 0;
	m_TileSize = 0;
	m_PixelMemSize = 0;
	m_AllocMode = 0;
	m_pMem = NULL;
}

CzMap::~CzMap(void) {
}

bool CzMap::Create() {
	return CzVar::Create(VarID_map);
}
bool CzMap::CreateShared(const char* lpVarName) {
	return CzVar::CreateShared(VarID_map, lpVarName);
}
bool CzMap::Attach(ZVAR hVar) {

	// if already attached, make sure we close the prev direct interface or lese risk leaking the scanline table
	if(m_hVar && m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}

	return CzVar::Attach(hVar);
}
void CzMap::Release() {

	// if already attached, make sure we close the prev direct interface or lese risk leaking the scanline table
	if(m_hVar && m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}

	return CzVar::Release();
}

bool CzMap::Init(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag) {
	if(!m_hVar)	return false;

	return zmap_Init(m_hVar, nx, ny, MapTypeID, HorizScale, WrapFlag);
}
bool CzMap::Init2(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, unsigned long AllocMode, long TileSize) {
	if(!m_hVar)	return false;

	return zmap_Init2(m_hVar, nx, ny, MapTypeID, HorizScale, WrapFlag, AllocMode, TileSize);
}
bool CzMap::Free() {
	if(!m_hVar)	return false;

	// if already attached, make sure we close the prev direct interface or lese risk leaking the scanline table
	if(m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}

	return zmap_Free(m_hVar);
}
long CzMap::GetMapType() {
	if(!m_hVar)	return 0;

	return zmap_GetMapType(m_hVar);
}
long CzMap::GetPixelSize() {
	if(!m_hVar)	return 0;

	return zmap_GetPixelSize(m_hVar);
}
bool CzMap::GetWrapFlag() {
	if(!m_hVar)	return false;

	return zmap_GetWrapFlag(m_hVar);
}
bool CzMap::SetWrapFlag(bool WrapFlag) {
	if(!m_hVar)	return false;

	return zmap_SetWrapFlag(m_hVar, WrapFlag);
}
bool CzMap::GetBusyFlag() {
	if(!m_hVar)	return false;

	return zmap_GetBusyFlag(m_hVar);
}
bool CzMap::SetBusyFlag(bool BusyFlag) {
	if(!m_hVar)	return false;

	return zmap_SetBusyFlag(m_hVar, BusyFlag);
}
long CzMap::nx() {

	if(!m_hVar)	return 0;

	return zmap_GetWidth(m_hVar);
}
long CzMap::ny() {

	if(!m_hVar)	return 0;

	return zmap_GetHeight(m_hVar);
}
__int64 CzMap::nPixels() {

	if(!m_hVar)	return 0;

	return (__int64)nx() * (__int64)ny();
}
bool CzMap::GetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(m_Contiguous) {
			int k = x + y * m_nx;
			memcpy(pValue, (char*)m_pMem + k * m_PixelMemSize, m_PixelMemSize);
			return true;
		} else {
			memcpy(pValue, ((char**)m_pMem)[y] + x * m_PixelMemSize, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;

	return zmap_GetPixel(m_hVar, x, y, pValue);
}
bool CzMap::GetPixel(__int64 k, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(m_Contiguous) {
			memcpy(pValue, (char*)m_pMem + k * m_PixelMemSize, m_PixelMemSize);
			return true;
		} else {
			int x = k%m_nx;
			int y = k/m_nx;

			memcpy(pValue, ((char**)m_pMem)[y] + x * m_PixelMemSize, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;
	
	long tnx = nx();
	if(tnx<=0)	return false;
	long x = (long)(k%(__int64)tnx);
	long y = (long)(k/(__int64)tnx);

	return GetPixel(x, y, pValue);
}
bool CzMap::SetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(m_Contiguous) {
			int k = x + y * m_nx;
			memcpy((char*)m_pMem + k * m_PixelMemSize, pValue, m_PixelMemSize);
			return true;
		} else {
			memcpy(((char**)m_pMem)[y] + x * m_PixelMemSize, pValue, m_PixelMemSize);
			return true;
		}
	}

	if(!m_hVar)	return false;

	return zmap_SetPixel(m_hVar, x, y, pValue);
}
bool CzMap::SetPixel(__int64 k, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(m_Contiguous) {
			memcpy((char*)m_pMem + k * m_PixelMemSize, pValue, m_PixelMemSize);
			return true;
		} else {
			int x = k%m_nx;
			int y = k/m_nx;

			memcpy(((char**)m_pMem)[y] + x * m_PixelMemSize, pValue, m_PixelMemSize); // scanlines
			return true;
		}
	}

	if(!m_hVar)	return false;
	
	long tnx = nx();
	if(tnx<=0)	return false;
	long x = (long)(k%(__int64)tnx);
	long y = (long)(k/(__int64)tnx);

	return SetPixel(x, y, pValue);
}
bool CzMap::GetMinMaxAlt(float& minval, float& maxval) {
	if(!m_hVar)	return false;

	return zmap_GetMinMaxAlt(m_hVar, minval, maxval);
}
bool CzMap::SetMinMaxAlt(float& minval, float& maxval) {
	if(!m_hVar)	return false;

	return zmap_SetMinMaxAlt(m_hVar, minval, maxval);
}
float CzMap::GetHorizScale() {
	if(!m_hVar)	return 1.0; // safe value, avoids div by zero

	return zmap_GetHorizScale(m_hVar);
}
bool CzMap::SetHorizScale(float scale) {
	if(!m_hVar)	return false;

	return zmap_SetHorizScale(m_hVar, scale);
}
bool CzMap::GetMosaicFlag() {
	if(!m_hVar)	return false;

	return zmap_GetMosaicFlag(m_hVar);
}
long CzMap::GetTileSize() {
	if(!m_hVar)	return 0;

	return zmap_GetTileSize(m_hVar);
}
long CzMap::GetTileCountX() {
	if(!m_hVar)	return 0;

	return zmap_GetTileCountX(m_hVar);
}
long CzMap::GetTileCountY() {
	if(!m_hVar)	return 0;

	return zmap_GetTileCountY(m_hVar);
}
bool CzMap::GetFlag(long FlagID) {
	if(!m_hVar)	return false;

	return zmap_GetFlag(m_hVar, FlagID);
}
bool CzMap::SetFlag(long FlagID, bool FlagValue) {
	if(!m_hVar)	return false;

	return zmap_SetFlag(m_hVar, FlagID, FlagValue);
}
unsigned long CzMap::GetAllocMode() {
	if(!m_hVar)	return 0;

	return zmap_GetAllocMode(m_hVar);
}
ZLIST CzMap::GetSettingsList() {
	if(!m_hVar)	return NULL;

	return zmap_GetSettingsList(m_hVar);
}
bool CzMap::SaveFile(const char* lpFileName, ZFORMAT hFormat, bool SetFlags, bool ShowProgress) {
	if(!m_hVar)	return false;

	return zmap_SaveFile(m_hVar, lpFileName, hFormat, SetFlags, ShowProgress);
}
bool CzMap::LoadFile(const char* lpFileName, long MapTypeID, ZFORMAT hFormat, bool SetFlags, bool ShowProgress) {
	if(!m_hVar)	return false;

	return zmap_LoadFile(m_hVar, lpFileName, MapTypeID, hFormat, SetFlags, ShowProgress);
}
bool CzMap::GetNameInProject(ZVAR hStr) {
	if(!m_hVar)	return false;

	return zmap_GetNameInProject(m_hVar, hStr);
}

bool CzMap::InitMosaic(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName) {
	if(!m_hVar)	return false;

	return zmap_InitMosaic(m_hVar, nx, ny, MapTypeID, HorizScale, WrapFlag, lpMosaicFileName, TileSize, hFormat, lpProjMapName);
}
bool CzMap::LoadMosaic(const char* lpMosaicFileName) {
	if(!m_hVar)	return false;

	return zmap_LoadMosaic(m_hVar, lpMosaicFileName);
}
bool CzMap::SaveMosaic() {
	if(!m_hVar)	return false;

	return zmap_SaveMosaic(m_hVar);
}

ZFORMAT CzMap::GetFormat() {
	if(!m_hVar)	return 0;

	return zmap_GetFormat(m_hVar);
}
bool CzMap::SetFormat(ZFORMAT hFormat) {
	if(!m_hVar)	return false;

	return zmap_SetFormat(m_hVar, hFormat);
}
const char* CzMap::GetFilename() {
	if(!m_hVar)	return 0;

	return zmap_GetFilename(m_hVar);
}
bool CzMap::CombineMosaic() {
	if(!m_hVar)	return 0;

	return zmap_CombineMosaic(m_hVar);
}
bool CzMap::SplitToMosaic(const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName) {
	if(!m_hVar)	return 0;

	return zmap_SplitToMosaic(m_hVar, lpMosaicFileName, TileSize, hFormat, lpProjMapName);
}
bool CzMap::GetMosaicTile(long ti, long tj, CzMapTile& tile) {
	if(!m_hVar)	return false;

	return tile.GetMapTile(m_hVar, ti, tj);
}
bool CzMap::SaveMosaicAs(const char* lpFileName) {
	if(!m_hVar)	return 0;

	return zmap_SaveMosaicAs(m_hVar, lpFileName);
}
bool CzMap::ExportMap(const char* lpFileName, ZFORMAT hFormat, long nx, long ny) {
	if(!m_hVar)	return 0;

	return zmap_ExportMap(m_hVar, lpFileName, hFormat, nx, ny);
}
bool CzMap::ExportMosaic(const char* lpFileName, ZFORMAT hFormat, long nx, long ny, long TileSize, const char* lpProjMapName) {
	if(!m_hVar)	return 0;

	return zmap_ExportMosaic(m_hVar, lpFileName, hFormat, nx, ny, TileSize, lpProjMapName);
}
void* CzMap::GetDataPtr() {
	if(!m_hVar)	return NULL;
	
	return zmap_GetDataPtr(m_hVar);
}
void* CzMap::GetScanlinePtr(long j) {
	if(!m_hVar)	return NULL;


	return zmap_GetScanlinePtr(m_hVar, j);
}
void** CzMap::GetScanlineArray() {
	if(!m_hVar)	return NULL;
	
	return zmap_GetScanlineArray(m_hVar);
}
void* CzMap::GetAt(long x, long y) {
	if(!m_hVar)	return NULL;
	if(MAPALLOC_RAM!=GetAllocMode()) return NULL;
	if(x<0 || x>=nx())	return NULL;
	if(y<0 || y>=ny())	return NULL;

	void* pLine = NULL;
	if(!m_DirectInterfaceFlag) {
		pLine = zmap_GetScanlinePtr(m_hVar, y);
	} else {
		if(m_Contiguous) {
			pLine = (void*)( (char*)m_pMem + m_PixelMemSize * m_nx * y);
		} else {
			pLine = ((void**)m_pMem)[y];
		}
	}
	
	if(!pLine)	return NULL;
	
	return (char*)pLine + x * m_PixelMemSize;
}
bool CzMap::IsContiguous() {
	if(!m_hVar)	return NULL;
	
	return zmap_IsContiguous(m_hVar);
}
bool CzMap::MakeContiguous() {
	if(!m_hVar)	return NULL;

	if(m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}
	
	return zmap_MakeContiguous(m_hVar);
}

bool CzMap::AsContiguousMap(CzMap& dest) {

	if(!m_hVar) {
		zeoReportError("CzMap::AsContiguousMap error:\r\n - no map attached");
		return false;
	}

	bool OldDirectInterface = m_DirectInterfaceFlag;
	if(!m_DirectInterfaceFlag) {
		if(!OpenDirectInterface()) {
			zeoReportError("CzMap::AsContiguousMap error:\r\n - cannot open direct interface");
			return false;
		}
	}

	if(dest.nPixels()) {
		if(dest.nx()!=m_ny || dest.ny()!=m_nx || dest.GetMapType()!=GetMapType()) {
			dest.Free();
		} else
		if(dest.GetAllocMode()!=MAPALLOC_RAM || !dest.IsContiguous()) {
			dest.Free();
		}
	}
	if(!dest.nPixels()) {
		if(!dest.Init2(m_nx, m_ny, GetMapType(), GetHorizScale(), GetWrapFlag(), MAPALLOC_RAM_CONTIGUOUS)) {

			zeoReportError("CzMap::AsContiguousMap error:\r\n - cannot initialise dest map");


			// cleanup
			if(!OldDirectInterface)	CloseDirectInterface();
			return false;
		}
	}


	// and open a direct interface to mem
	if(!dest.OpenDirectInterface()) {

		zeoReportError("CzMap::AsContiguousMap error:\r\n - cannot open direct interface with dest map");

		// cleanup
		if(!OldDirectInterface)	CloseDirectInterface();
		return false;
	}

	void* pDestMapData = dest.GetDataPtr(); // dest is contiguous, so this is OK
	if(!pDestMapData) {

		zeoReportError("CzMap::AsContiguousMap error:\r\n - invalid pointer to dest map data");

		// cleanup
		if(!OldDirectInterface)	CloseDirectInterface();
		return false;
	}

	long LineLen = m_PixelMemSize * m_nx;

	for(int j=0; j<m_ny; j++) {
		void* pSrc = GetScanlinePtr(j);
		if(!pSrc) {
			zeoReportError("CzMap::AsContiguousMap error:\r\n - cannot get source line ptr");

			// cleanup
			if(!OldDirectInterface)	CloseDirectInterface();
			return false;
		}

		void* pDest = (void*)( (char*)pDestMapData + j * LineLen);
		
		memcpy(pDest, pSrc, LineLen);
	}
	
	// cleanup
	if(!OldDirectInterface)	CloseDirectInterface();

	return true;
}
bool CzMap::LinInterp(double dx, double dy, void* pValue) {
	if(!m_hVar)	return 0;

	return zmap_LinInterp(m_hVar, dx, dy, pValue);
}

bool CzMap::GenMipmaps(long ResStep, long MaxLevel, long TileSize) {
	if(!m_hVar)	return false;

	return zmap_GenMipmaps(m_hVar, ResStep, MaxLevel, TileSize);
}
bool CzMap::ClearMipmaps() {
	if(!m_hVar)	return false;

	return zmap_ClearMipmaps(m_hVar);
}
ZMAP CzMap::GetMipmapLevel(long MipLevel) {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapLevel(m_hVar, MipLevel);
}
long CzMap::GetMipmapResStep() {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapResStep(m_hVar);
}
long CzMap::GetMipmapMaxLevel() {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapMaxLevel(m_hVar);
}
long CzMap::nMipmaps() {
	if(!m_hVar)	return 0;

	return zmap_nMipmaps(m_hVar);
}
bool CzMap::AllocMipmaps(long MipIncr, long MaxMip) {
	if(!m_hVar)	return 0;

	return zmap_AllocMipmaps(m_hVar, MipIncr, MaxMip);
}
bool CzMap::GetMipmapLevel2(long MipID, long &MipRes, ZMAP* ppMipMap) {
	if(!m_hVar)	return 0;

	return zmap_GetMipmapLevel2(m_hVar, MipID, MipRes, ppMipMap);
}

// direct interface
bool CzMap::OpenDirectInterface() {
	
	if(!m_hVar) {
		zeoReportError("CzMap::OpenDirectInterface error:\r\n - invalid var handle");
		return false;
	}

	if(m_DirectInterfaceFlag) {
		zeoWriteToLog("CzMap::OpenDirectInterface warning:\r\n - already open");
		return true;
	}

	m_AllocMode = GetAllocMode();
	if(MAPALLOC_RAM!=m_AllocMode) {
		// fail quietly; you can't do direct access on a mosaic; use zmap_GetMosaicTile & friends
		return false;
	}

	m_nx = nx();
	m_ny = ny();

	m_Contiguous = IsContiguous();

	if(m_Contiguous) {
		m_pMem = GetDataPtr();

		if(!m_pMem) {
			zeoReportError("CzMap::OpenDirectInterface error:\r\n - cannot get mem handle");
			return false;
		}
	} else {

		m_pMem = (void*)GetScanlineArray();
		if(!m_pMem) {
			zeoReportError("CzMap::OpenDirectInterface error:\r\n - cannot allocate scanline array");
			return false;
		}
	}
	
	m_TileSize = GetTileSize();
	m_nTilesX = GetTileCountX();
	m_nTilesY = GetTileCountY();
	m_nPixels = nPixels();
	m_PixelMemSize = GetPixelSize();

	m_DirectInterfaceFlag = true;

	return true;
}
void CzMap::CloseDirectInterface() {
	if(!m_DirectInterfaceFlag) {
		return;
	}

	m_pMem = NULL;

	m_DirectInterfaceFlag = false;
}
