#include "vectors.h"
#include <math.h>

// setting/getting values

vector3f VectorSet3f(float x, float y, float z) {
	vector3f v;
	v.x = x;
	v.y = y;
	v.z = z;
	return v;
}
vector3f VectorSet3f(vector3d v) { // double to float
	vector3f v2;
	v2.x = (float)v.x;
	v2.y = (float)v.y;
	v2.z = (float)v.z;
	return v2;
}


vector3d VectorSet3d(double x, double y, double z) {
	vector3d v;
	v.x = x;
	v.y = y;
	v.z = z;
	return v;
}
vector3d VectorSet3d(vector3f v) { // float to double
	vector3d v2;
	v2.x = (double)v.x;
	v2.y = (double)v.y;
	v2.z = (double)v.z;
	return v2;
}

vector4f VectorSet4f(float a, float b, float c, float d) {
	vector4f v;
	v.a = a;
	v.b = b;
	v.c = c;
	v.d = d;
	return v;
}

// addition, subtraction

vector3f VectorAdd3f(vector3f a, vector3f b) {
	vector3f v;
	v.x = a.x + b.x;
	v.y = a.y + b.y;
	v.z = a.z + b.z;
	return v;
}
vector3f VectorSub3f(vector3f a, vector3f b) {
	vector3f v;
	v.x = a.x - b.x;
	v.y = a.y - b.y;
	v.z = a.z - b.z;
	return v;
}

vector3d VectorAdd3d(vector3d a, vector3d b) {
	vector3d v;
	v.x = a.x + b.x;
	v.y = a.y + b.y;
	v.z = a.z + b.z;
	return v;
}
vector3d VectorSub3d(vector3d a, vector3d b) {
	vector3d v;
	v.x = a.x - b.x;
	v.y = a.y - b.y;
	v.z = a.z - b.z;
	return v;
}

// scaling, negation

vector3f VectorScale3f(vector3f v, float f) {
	vector3f v2;
	v2.x = v.x * f;
	v2.y = v.y * f;
	v2.z = v.z * f;
	return v2;
}

vector3f VectorNegate3f(vector3f v) {
	vector3f v2;
	v2.x =- v.x;
	v2.y =- v.y;
	v2.z =- v.z;
	return v2;
}
vector3f VectorScale3fv(vector3f v, vector3f scale) {
	vector3f v2;
	v2.x = v.x * scale.x;
	v2.y = v.y * scale.y;
	v2.z = v.z * scale.z;
	return v2;
}

vector3d VectorScale3d(vector3d v, double d) {
	vector3d v2;
	v2.x = v.x * d;
	v2.y = v.y * d;
	v2.z = v.z * d;
	return v2;
}

vector3d VectorNegate3d(vector3d v) {
	vector3d v2;
	v2.x =- v.x;
	v2.y =- v.y;
	v2.z =- v.z;
	return v2;
}

// length, normalisation

double VectorNorm3f(vector3f v) {
	return ScalarProd3f(v, v);
}
double VectorLength3f(vector3f v) {
	return sqrt(VectorNorm3f(v));
}
vector3f VectorNormalise3f(vector3f v) {
	return VectorScale3f(v, 1.0f / (float)VectorLength3f(v));
}

double VectorNorm3d(vector3d v) {
	return ScalarProd3d(v, v);
}
double VectorLength3d(vector3d v) {
	return sqrt(VectorNorm3d(v));
}
vector3d VectorNormalise3d(vector3d v) {
	return VectorScale3d(v, 1.0 / VectorLength3d(v));
}

// scalar and cross-product

double ScalarProd3f(vector3f a, vector3f b) {
	return (a.x * b.x + a.y * b.y + a.z * b.z);
}
vector3f CrossProd3f(vector3f a, vector3f b) {
	vector3f v;
	v.x =     a.y * b.z - b.y * a.z;
	v.y = - ( a.x * b.z - b.x * a.z);
	v.z =     a.x * b.y - b.x * a.y;
	return v;
}


double ScalarProd3d(vector3d a, vector3d b) {
	return (a.x * b.x + a.y * b.y + a.z * b.z);
}
vector3d CrossProd3d(vector3d a, vector3d b) {
	vector3d v;
	v.x =     a.y * b.z - b.y * a.z;
	v.y = - ( a.x * b.z - b.x * a.z);
	v.z =     a.x * b.y - b.x * a.y;
	return v;
}

// matrix transforms

vector3d VectorTransform3d(vector3d v, matrix3d m) {
	vector3d v2;

	v2.x = ScalarProd3d(m.i, v);
	v2.y = ScalarProd3d(m.j, v);
	v2.z = ScalarProd3d(m.k, v);

	return v2;
}

// matrix math

matrix3d MatrixIdentity3d() {
	matrix3d m;
	m.i = VectorSet3d(1, 0, 0);
	m.j = VectorSet3d(0, 1, 0);
	m.k = VectorSet3d(0, 0, 1);
	return m;
}
matrix3f MatrixIdentity3f() {
	matrix3f m;
	m.i = VectorSet3f(1, 0, 0);
	m.j = VectorSet3f(0, 1, 0);
	m.k = VectorSet3f(0, 0, 1);
	return m;
}
matrix3d MatrixSet3d(matrix3f m) {
	matrix3d m2;
	m2.i = VectorSet3d(m.i);
	m2.j = VectorSet3d(m.j);
	m2.k = VectorSet3d(m.k);
	return m2;
}

vector3d MatrixRow3d(matrix3d m, int RowID) {
	switch(RowID) {
	case 0:	return m.i;
	case 1:	return m.j;
	case 2:	return m.k;
	}
	return VectorSet3d(0, 0, 0);
}
vector3d MatrixColumn3d(matrix3d m, int ColumnID) {
	switch(ColumnID) {
	case 0:	return VectorSet3d(m.i.x, m.j.x, m.k.x);
	case 1:	return VectorSet3d(m.i.y, m.j.y, m.k.y);
	case 2:	return VectorSet3d(m.i.z, m.j.z, m.k.z);
	}
	return VectorSet3d(0, 0, 0);
}

matrix3d MatrixProd3d(matrix3d a, matrix3d b) {
	matrix3d c;

	c.i.x = ScalarProd3d(a.i, MatrixColumn3d(b, 0));
	c.i.y = ScalarProd3d(a.i, MatrixColumn3d(b, 1));
	c.i.z = ScalarProd3d(a.i, MatrixColumn3d(b, 2));

	c.j.x = ScalarProd3d(a.j, MatrixColumn3d(b, 0));
	c.j.y = ScalarProd3d(a.j, MatrixColumn3d(b, 1));
	c.j.z = ScalarProd3d(a.j, MatrixColumn3d(b, 2));

	c.k.x = ScalarProd3d(a.k, MatrixColumn3d(b, 0));
	c.k.y = ScalarProd3d(a.k, MatrixColumn3d(b, 1));
	c.k.z = ScalarProd3d(a.k, MatrixColumn3d(b, 2));

	return c;
}

matrix3d  MatrixRotateX3d(matrix3d m, float angle) {
	matrix3d r;

	r.i = VectorSet3d(1, 0, 0);
	r.j = VectorSet3d(0, cos(angle), -sin(angle));
	r.k = VectorSet3d(0, sin(angle), cos(angle));

	return MatrixProd3d(r, m);
}
matrix3d  MatrixRotateY3d(matrix3d m, float angle) {
	matrix3d r;
	r.i = VectorSet3d(cos(angle), 0, -sin(angle));
	r.j = VectorSet3d(0, 1, 0);
	r.k = VectorSet3d(sin(angle), 0, cos(angle));
	
	return MatrixProd3d(r, m);
}
matrix3d  MatrixRotateZ3d(matrix3d m, float angle) {
	matrix3d r;
	r.i = VectorSet3d(cos(angle), -sin(angle), 0);
	r.j = VectorSet3d(sin(angle), cos(angle), 0);
	r.k = VectorSet3d(0, 0, 1);

	return MatrixProd3d(r, m);
}

matrix3d MatrixFromAE3d(float azim, float elev) {
	return MatrixRotateX3d(MatrixRotateZ3d(MatrixIdentity3d(), azim), -elev);
}

matrix3d MatrixFromAER3d(float azim, float elev, float roll) {
	return MatrixRotateY3d(MatrixRotateX3d(MatrixRotateZ3d(MatrixIdentity3d(), azim), -elev), roll);
}