#include "Zeolite.h"
#include "zMeshFaceGroup.h"

#include <zBuffer.h>

CzMeshFaceGroup::CzMeshFaceGroup(void) {
	
	m_hMeshMode = NULL;

	m_hVertIndices = NULL;
	m_hNormIndices = NULL;
	m_hTexIndices = NULL;

	m_hMtlName = NULL;

	m_hInfoEx = NULL;
}

CzMeshFaceGroup::~CzMeshFaceGroup(void) {
}

bool CzMeshFaceGroup::CreateData(ZLIST hTarget) {

	if(hTarget) {
		if(!CzVar::Attach(hTarget)) {
			zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - call to CzVar::Attach failed on hTarget");
			return false;
		}
	}

	if(!m_hVar) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - face group variable was not created");
		return false;
	}

	if(zlist_nItems(m_hVar)) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - face group is not initialised");
		return false;
	}

	// root level items follow
	int fgVer = 1;
	if(!zvar_SetValue(zlist_CreateItem(m_hVar, VarID_int, "fgVer"), &fgVer)) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot set face group version");
		return false;
	}

	int MeshMode = 0;
	if(!zvar_SetValue((m_hMeshMode = zlist_CreateItem(m_hVar, VarID_int, "MeshMode")), &MeshMode)) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot set face group mesh mode");
		return false;
	}

	if(! (m_hVertIndices = zlist_CreateItem(m_hVar, VarID_buffer, "VertIndexBuf"))) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot create vert index buffer");
		return false;
	}
	if(! (m_hNormIndices = zlist_CreateItem(m_hVar, VarID_buffer, "NormIndexBuf"))) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot create norm index buffer");
		return false;
	}
	if(! (m_hTexIndices = zlist_CreateItem(m_hVar, VarID_buffer, "TexIndexBuf"))) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot create tex index buffer");
		return false;
	}

	if(! (m_hMtlName = zlist_CreateItem(m_hVar, VarID_string, "MtlName"))) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot create material name");
		return false;
	}

	if(! (m_hInfoEx = zlist_CreateItem(m_hVar, VarID_varlist, "InfoEx"))) {
		zeoReportError("CzMeshFaceGroup::CreateData error:\r\n - cannot create InfoEx");
		return false;
	}

	return true;
}

bool CzMeshFaceGroup::Attach(ZVAR hVar) {
	
	if(!CzVar::Attach(hVar)) {
		zeoReportError("CzMeshFaceGroup::Attach error:\r\n - call to CzVar::Attach failed");
		return false;
	}

	int argid = 0;

	// get mesh version
	int fgVer = -1;
	if(!zvar_GetValueEx(zlist_GetItemI(m_hVar, argid++), VarID_int, &fgVer)) {
		zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get face group version");

		Release();
		return false;
	}

	switch(fgVer) {
	case -1:
		zeoReportError("CzMeshFaceGroup::Attach error:\r\n - invalid face group version");
		Release();
		return false;
	default:
		zeoReportError("CzMeshFaceGroup::Attach error:\r\n - unsupported face group version");
		Release();
		return false;
	case 1:
		{
			
			if(!zvar_IsType((m_hMeshMode = zlist_GetItemI(m_hVar, argid++)), VarID_int)) {
				zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get face group mesh mode");
				return false;
			}

			if(!zvar_IsType((m_hVertIndices = zlist_GetItemI(m_hVar, argid++)), VarID_buffer)) {
				zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get vert index buffer");
				return false;
			}

			if(!zvar_IsType((m_hNormIndices = zlist_GetItemI(m_hVar, argid++)), VarID_buffer)) {
				zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get norm index buffer");
				return false;
			}

			if(!zvar_IsType((m_hTexIndices = zlist_GetItemI(m_hVar, argid++)), VarID_buffer)) {
				zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get tex index buffer");
				return false;
			}

			if(!zvar_IsType((m_hMtlName = zlist_GetItemI(m_hVar, argid++)), VarID_string)) {
				zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get material name");
				return false;
			}

			if(!zvar_IsType((m_hInfoEx = zlist_GetItemI(m_hVar, argid++)), VarID_varlist)) {
				zeoReportError("CzMeshFaceGroup::Attach error:\r\n - cannot get InfoEx");
				return false;
			}

		}
		break;
	}

	return true;
}

long CzMeshFaceGroup::GetMeshMode() {
	long rval = -1;
	zvar_GetValue(m_hMeshMode, &rval);
	return rval;
}
bool CzMeshFaceGroup::SetMeshMode(long MeshMode) {
	switch(MeshMode) {
	default:
		return false;
	case MESHMODE_POINTS:
	case MESHMODE_LINES:
	case MESHMODE_LINE_LOOP:
	case MESHMODE_LINE_STRIP:
	case MESHMODE_TRIANGLES:
	case MESHMODE_TRIANGLE_STRIP:
	case MESHMODE_TRIANGLE_FAN:
	case MESHMODE_QUADS:
	case MESHMODE_QUAD_STRIP:
	case MESHMODE_POLYGON:
		// valid mesh mode (from OpenGL glBegin modes)
		break;
	}
	return zvar_SetValue(m_hMeshMode,  &MeshMode);
}

const char* CzMeshFaceGroup::GetMaterialName() {
	return zstr_GetText(m_hMtlName);
}
bool CzMeshFaceGroup::SetMaterialName(const char* lpMtlName) {
	return zstr_SetText(m_hMtlName, lpMtlName);
}

bool CzMeshFaceGroup::Init(long MeshMode, const char* lpMtlName, ZBUF hVertIndexBuf, ZBUF hNormIndexBuf, ZBUF hTexIndexBuf) {

	if(!m_hVar) {
		zeoReportError("CzMeshFaceGroup::Init error:\r\n - mesh variable was not created");
		return false;
	}

	if(!zvar_SetValueEx(m_hMeshMode, VarID_int, &MeshMode)) {
		zeoReportError("CzMeshFaceGroup::Init error:\r\n - cannot set mesh mode");
		return false;
	}

	if(!zstr_SetText(m_hMtlName, lpMtlName)) {
		zeoReportError("CzMeshFaceGroup::Init error:\r\n - cannot set material name");
		return false;
	}

	if(!SetVertIndexBuf(hVertIndexBuf)) {
		zeoReportError("CzMeshFaceGroup::Init error:\r\n - call to SetVertIndexBuf failed");
		return false;
	}
	if(!SetNormIndexBuf(hNormIndexBuf)) {
		zeoReportError("CzMeshFaceGroup::Init error:\r\n - call to SetNormIndexBuf failed");
		return false;
	}
	if(!SetTexIndexBuf(hTexIndexBuf)) {
		zeoReportError("CzMeshFaceGroup::Init error:\r\n - call to SetTexIndexBuf failed");
		return false;
	}

	return true;
}

bool CzMeshFaceGroup::SetVertIndexBuf(ZBUF hBuf) {
	CzBuffer b;
	if(!b.Attach(m_hVertIndices)) {
		zeoReportError("CzMeshFaceGroup::SetVertIndexBuf error:\r\n - call to CzBuffer:Attach failed");
		return false;
	}
	if(b.nItems())	b.Free();
	if(hBuf) {
		if(VarID_int!=zbuffer_ItemType(hBuf)) {
			zeoReportError("CzMeshFaceGroup::SetVertIndexBuf error:\r\n - invalid type for vertex buffer");
			return false;
		}
		if(!b.CopyValue(hBuf)) {
			zeoReportError("CzMeshFaceGroup::SetVertIndexBuf error:\r\n - call to CzBuffer:CopyValue failed");
			return false;
		}

		// now free norm and tex buffers if different size
		long nItems = zbuffer_nItems(m_hVertIndices);
		if(zbuffer_nItems(m_hNormIndices) && nItems!=zbuffer_nItems(m_hNormIndices)) {
			zbuffer_Free(m_hNormIndices);
		}
		if(zbuffer_nItems(m_hTexIndices) && nItems!=zbuffer_nItems(m_hTexIndices)) {
			zbuffer_Free(m_hTexIndices);
		}
	}

	return true;
}
bool CzMeshFaceGroup::SetNormIndexBuf(ZBUF hBuf) {

	CzBuffer b;
	if(!b.Attach(m_hNormIndices)) {
		zeoReportError("CzMeshFaceGroup::SetNormIndexBuf error:\r\n - call to CzBuffer:Attach failed");
		return false;
	}
	if(b.nItems())	b.Free();
	if(hBuf) {
		if(VarID_int!=zbuffer_ItemType(hBuf)) {
			zeoReportError("CzMeshFaceGroup::SetNormIndexBuf error:\r\n - invalid type for normal buffer");
			return false;
		}
		if(zbuffer_nItems(m_hVertIndices)!=zbuffer_nItems(hBuf)) {
			// norm buffer must be same size as vert buffer
			zeoReportError("CzMeshFaceGroup::SetNormIndexBuf error:\r\n - normal index buffer must be, but is not, the same length as the vertex index buffer");
			return false;
		}
		if(!b.CopyValue(hBuf)) {
			zeoReportError("CzMeshFaceGroup::SetNormIndexBuf error:\r\n - call to CzBuffer:CopyValue failed");
			return false;
		}
	}

	return true;
}
bool CzMeshFaceGroup::SetTexIndexBuf(ZBUF hBuf) {

	CzBuffer b;
	if(!b.Attach(m_hTexIndices)) {
		zeoReportError("CzMeshFaceGroup::SetTexIndexBuf error:\r\n - call to CzBuffer:Attach failed");
		return false;
	}
	if(b.nItems())	b.Free();
	if(hBuf) {
		if(VarID_int!=zbuffer_ItemType(hBuf)) {
			zeoReportError("CzMeshFaceGroup::SetTexIndexBuf error:\r\n - invalid type for texture buffer");
			return false;
		}
		if(zbuffer_nItems(m_hVertIndices)!=zbuffer_nItems(hBuf)) {
			// tex buffer must be same size as vert buffer
			zeoReportError("CzMeshFaceGroup::SetTexIndexBuf error:\r\n - texture index buffer must be, but is not, the same length as the vertex index buffer");
			return false;
		}
		if(!b.CopyValue(hBuf)) {
			zeoReportError("CzMeshFaceGroup::SetTexIndexBuf error:\r\n - call to CzBuffer:CopyValue failed");
			return false;
		}
	}

	return true;
}