#include <zeolite.h>
#include "zMeshTransforms.h"
#include "zMesh.h"
#include "zMeshFaceGroup.h"
#include "vectors.h"
#include <zbuffer.h>

bool zmesh_SwapAxesToXrightZforwardYup(CzMesh* pMesh) {

	if(!pMesh) {
		zeoReportError("zmesh_SwapAxesToXrightZforwardYup error:\r\n - invalid mesh handle");
		return false;
	}

	CzBuffer VertBuf;
	if(!VertBuf.Attach(pMesh->GetVertBuf())) {
		zeoReportError("zmesh_SwapAxesToXrightZforwardYup error:\r\n - cannot get vertex buffer");
		return false;
	}

	// vertices: invert Y and swap with Z (also swap origin offset for yaxis)
	for(int i=0; i<VertBuf.nItems(); i++) {
		vector3f v;
		VertBuf.GetValue(i, &v);
		vector3f v2 = VectorSet3f(v.x, v.z, -v.y);
		VertBuf.SetValue(i, &v2);
	}
	

	// normals: invert Y and swap with Z
	CzBuffer NormBuf;
	if(!NormBuf.Attach(pMesh->GetNormBuf())) {
		zeoReportError("zmesh_SwapAxesToXrightZforwardYup error:\r\n - cannot get normal buffer");
		return false;
	}
	for(int i=0; i<NormBuf.nItems(); i++) {
		vector3f v;
		NormBuf.GetValue(i, &v);
		vector3f v2 = VectorSet3f(v.x, v.z, -v.y);
		NormBuf.SetValue(i, &v2);
	}
	
	// finally, reverse the face winding to maintain CCW winding
	if(!zmesh_ReverseWindFaces(pMesh)) {
		zeoReportError("zmesh_SwapAxesToXrightZforwardYup error:\r\n - call to ReverseWindFaces failed");
		return false;
	}

	return true;
}

bool zmesh_Offset(CzMesh* pMesh, float x, float y, float z) {

	if(!pMesh) {
		zeoReportError("zmesh_Offset error:\r\n - invalid mesh handle");
		return false;
	}

	CzBuffer VertBuf;
	if(!VertBuf.Attach(pMesh->GetVertBuf())) {
		zeoReportError("zmesh_Offset error:\r\n - cannot get vertex buffer");
		return false;
	}
	for(int v=0; v<VertBuf.nItems(); v++) {
		vector3f* pv3 = (vector3f*)VertBuf.GetAt(v);
		pv3->x +=x;
		pv3->y +=y;
		pv3->z +=z;
	}

	return true;
}

bool zmesh_ReverseWindFaces(CzMesh* pMesh) {

	if(!pMesh) {
		zeoReportError("zmesh_ReverseWindFaces error:\r\n - invalid mesh handle");
		return false;
	}

	for(int f=0; f<pMesh->nFaceGroups(); f++) {

		CzMeshFaceGroup fg;
		if(!pMesh->GetFaceGroup(f, fg)) {
			zeoReportError("zmesh_ReverseWindFaces error:\r\n - cannot get face group");
			return false;
		} 
	
		// vertices
		CzBuffer VertIndexBuf;
		if(VertIndexBuf.Attach(fg.GetVertIndexBuf())) {
			for(int i=0; i<VertIndexBuf.nItems(); i+=3) {
				int face[2] = {0, 0};
				VertIndexBuf.GetValue(i+1, &face[0]);
				VertIndexBuf.GetValue(i+2, &face[1]);
				VertIndexBuf.SetValue(i+1, &face[1]);
				VertIndexBuf.SetValue(i+2, &face[0]);
			}
		}

		// normals
		CzBuffer NormIndexBuf;
		if(NormIndexBuf.Attach(fg.GetNormIndexBuf())) {
			for(int i=0; i<NormIndexBuf.nItems(); i+=3) {
				int face[2] = {0, 0};
				NormIndexBuf.GetValue(i+1, &face[0]);
				NormIndexBuf.GetValue(i+2, &face[1]);
				NormIndexBuf.SetValue(i+1, &face[1]);
				NormIndexBuf.SetValue(i+2, &face[0]);
			}
		}

		// texcoords
		CzBuffer TexIndexBuf;
		if(TexIndexBuf.Attach(fg.GetTexIndexBuf())) {
			for(int i=0; i<TexIndexBuf.nItems(); i+=3) {
				int face[2] = {0, 0};
				TexIndexBuf.GetValue(i+1, &face[0]);
				TexIndexBuf.GetValue(i+2, &face[1]);
				TexIndexBuf.SetValue(i+1, &face[1]);
				TexIndexBuf.SetValue(i+2, &face[0]);
			}
		}
	}

	return true;
}
