#include <Zeolite.h>
#include "zPointList.h"
#include <zVar.h>
#include <zList.h>
#include <zProj.h>
#include <zmesh/vectors.h>

CzPointList::CzPointList() {
	m_Version = 1;
	m_hPointList = NULL;
	m_hChildList = NULL;
	m_Flags = 0;
	m_hOptions = NULL;
}

CzPointList::CzPointList(ZLIST hContainer) {
	m_Version = 1;
	m_hPointList = NULL;
	m_hChildList = NULL;
	m_Flags = 0;
	m_hOptions = NULL;

	Attach(hContainer);
}
	
CzPointList::~CzPointList() {
}


bool CzPointList::Create(ZLIST hContainer, unsigned long Flags) {

	if(hContainer) {
		if(VarID_varlist!=zvar_GetTypeID(hContainer)) {
			zeoReportError("CzPointList::Create error:\r\n - container is not a varlist");
			return false;
		}
		if(zlist_nItems(hContainer)) {
			zeoReportError("CzPointList::Create error:\r\n - container is not empty");
			return false;
		}
		if(!CzVar::Attach(hContainer)) {
			zeoReportError("CzPointList::Create error:\r\n - cannot attach container");
			return false;
		}
	} else {
		if(!CzVar::Create(VarID_varlist)) {
			zeoReportError("CzPointList::Create error:\r\n - cannot create container");
			return false;
		}
	}

	// apply default flag options if not given
	unsigned long xyFlags = Flags & ZPFLAG_XY_COORD_MASKALL;
	if(!xyFlags) {
		Flags = Flags|ZPFLAG_XY_COORD_REL;
	}
	unsigned long zFlags = Flags & ZPFLAG_Z_COORD_MASKALL;
	if(!zFlags) {
		Flags = Flags|ZPFLAG_Z_COORD_ABOVE_SEALEVEL;
	}
	unsigned long yprFlags = Flags & ZPFLAG_YPR_COORD_MASKALL;
	if(!yprFlags) {
		Flags = Flags|ZPFLAG_YPR_COORD_RAD;
	}



	CzList ls;
	if(!ls.Attach(m_hVar)) {
		zeoReportError("CzPointList::Create error:\r\n - cannot attach container as varlist");
		return false;
	}

	m_Version = 1;
	if(!ls.SetItemValueA("Version", VarID_uint, &m_Version)) {
		zeoReportError("CzPointList::Create error:\r\n - cannot set version");
		return false;
	}

	if(!( m_hPointList = ls.CreateItem(VarID_varlist, "PointList"))) {
		zeoReportError("CzPointList::Create error:\r\n - cannot create point list");
		return false;
	}

	if(!(m_hChildList = ls.CreateItem(VarID_varlist, "ChildList"))) {
		zeoReportError("CzPointList::Create error:\r\n - cannot create child list");
		return false;
	}

	m_Flags = Flags;
	if(!ls.SetItemValueA("Flags", VarID_uint, &m_Flags)) {
		zeoReportError("CzPointList::Create error:\r\n - cannot create flags");
		return false;
	}

	if(!(m_hOptions = ls.CreateItem(VarID_varlist, "Options"))) {
		zeoReportError("CzPointList::Create error:\r\n - cannot create options list");
		return false;
	}

 	return true;
}
bool CzPointList::CreateShared(const char* lpVarName, unsigned long Flags) {

	ZVAR hContainer = zvar_CreateShared(VarID_varlist, lpVarName);
	if(!hContainer) {
		zeoReportError("CzPointList::CreateShared error:\r\n - cannot create container");
		return false;
	}

	if(!Create(hContainer, Flags)) {
		zeoReportError("CzPointList::CreateShared error:\r\n - call to Create failed");
		return false;
	}

	return true;
}

bool CzPointList::Attach(ZLIST hPointList) {

	if(VarID_varlist!=zvar_GetTypeID(hPointList)) {
		zeoReportError("CzPointList::Attach error:\r\n - hPointList is not a varlist");
		return false;
	}

	if(!CzVar::Attach(hPointList)) {
		zeoReportError("CzPointList::Attach error:\r\n - call to CzVar::Attach failed");
		return false;
	}

	CzList ls;
	if(!ls.Attach(m_hVar)) {
		zeoReportError("CzPointList::Attach error:\r\n - call to CzList::Attach failed");
		return false;
	}

	m_Version = 0;
	if(!ls.GetItemValueI(0, VarID_uint, &m_Version)) { // version is always 0
		zeoReportError("CzPointList::Attach error:\r\n - cannot get version number");
		return false;
	}

	if(m_Version==1) {

		if(!(m_hPointList = ls.GetItem(1))) { // point list at 1
			zeoReportError("CzPointList::Attach error:\r\n - cannot get point list");
			return false;
		}

		if(! (m_hChildList = ls.GetItemI(2))) { // child list at 2
			zeoReportError("CzPointList::Attach error:\r\n - cannot get child list");
			return false;
		}

		m_Flags = 0;
		if(!ls.GetItemValue(3, VarID_uint, &m_Flags)) { // flags at 3
			zeoReportError("CzPointList::Attach error:\r\n - cannot get flags");
			return false;
		}

		if(! (m_hOptions = ls.GetItem(4))) { // options at 4
			zeoReportError("CzPointList::Attach error:\r\n - cannot get options list");
			return false;
		}
	} else {
		// unsupported version number
		zeoReportError("CzPointList::Attach error:\r\n - unsupported version number");
		return false;
	}

	return true;
}
bool CzPointList::AttachToProjectList() {

	ZLIST hProjSettings = zproj_GetSettingsList();
	if(!hProjSettings) {
		zeoReportError("CzPointList::AttachToProjectList error:\r\n - cannot get project settings list");
		return false;
	}

	ZLIST hContainer = zlist_GetItemA(hProjSettings, "PointList");
	if(hContainer) {
		if(!Attach(hContainer)) {
			zeoReportError("CzPointList::AttachToProjectList error:\r\n - cannot attach point list");
			return false;
		}
	} else {
		hContainer = zlist_CreateItem(hProjSettings, VarID_varlist, "PointList");
		if(!hContainer) {
			zeoReportError("CzPointList::AttachToProjectList error:\r\n - cannot create container");
			return false;
		}

		if(!Create(hContainer, NULL)) {
			zeoReportError("CzPointList::AttachToProjectList error:\r\n - call to Create failed");
			return false;
		}
	}
	
	return true;
}
void CzPointList::Release() {

	m_Version = 0;
	m_hPointList = NULL;
	m_hChildList = NULL;
	m_Flags = 0;
	m_hOptions = NULL;
	
	CzVar::Release();
}

//
// sublists
//

long CzPointList::nSubLists() {
	return zlist_nItems(m_hChildList);
}
long CzPointList::CreateSubList(const char* lpName, unsigned long Flags) {
	if(!m_hChildList) {
		zeoReportError("CzPointList::CreateSubList error:\r\n - list not initialised");
		return -1;
	}

	ZLIST hItem = zlist_CreateItem(m_hChildList, VarID_varlist, lpName);
	if(!hItem) {
		zeoReportError("CzPointList::CreateSubList error:\r\n - cannot create sublist");
		return -1;
	}
	long iItem = zlist_nItems(m_hChildList)-1;
	
	CzPointList pl;
	if(!pl.Create(hItem, Flags)) {
		zeoReportError("CzPointList::CreateSubList error:\r\n - call to Create failed for sublist");

		zlist_DelItemI(m_hChildList, iItem);
		return -1;
	}
	
	return iItem;
}
bool CzPointList::GetSubList(long index, CzPointList& SubList) {
	ZLIST hItem = zlist_GetItemI(m_hChildList, index);
	if(!hItem) {
		zeoReportError("CzPointList::GetSubList error:\r\n - cannot get sublist");
		return false;
	}

	if(!SubList.Attach(hItem)) {
		zeoReportError("CzPointList::GetSubList error:\r\n - cannot attach sublist");
		return false;
	}

	return true;
}
bool CzPointList::DeleteSubList(long index) {
	return zlist_DelItemI(m_hChildList, index);
}
bool CzPointList::DeleteSubLists() {
	return zlist_DelAll(m_hChildList);
}
long CzPointList::FindSubList(const char* lpName) {
	CzList ls;
	if(!ls.Attach(m_hChildList)) {
		zeoReportError("CzPointList::FindSubList error:\r\n - cannot attach sublist");
		return -1;
	}
	return ls.FindItemA(lpName);
}

//
// points
//

long CzPointList::nPoints() {
	return zlist_nItems(m_hPointList);
}
long CzPointList::CreatePoint(const char* lpName) {
	if(!m_hPointList) {
		zeoReportError("CzPointList::CreatePoint error:\r\n - list not initialised");
		return -1;
	}

	ZLIST hItem = zlist_CreateItem(m_hPointList, VarID_varlist, lpName);
	if(!hItem) {
		zeoReportError("CzPointList::CreatePoint error:\r\n - cannot create point");
		return -1;
	}

	// vector3d for pos
	zlist_CreateItem(hItem, VarID_vector3d, NULL);
	
	// vector3f for ypr (optional)
	if(m_Flags & ZPFLAG_STORE_YPR) {
		zlist_CreateItem(hItem, VarID_vector3d, NULL);
	}

	// uint for flags (optional)
	if(m_Flags & ZPFLAG_STORE_FLAGS) {
		zlist_CreateItem(hItem, VarID_uint, NULL);
	}

	// int for lparam (optional)
	if(m_Flags & ZPFLAG_STORE_PARAMI) {
		zlist_CreateItem(hItem, VarID_int, NULL);
	}

	// double for paramd (optional)
	if(m_Flags & ZPFLAG_STORE_PARAMD) {
		zlist_CreateItem(hItem, VarID_double, NULL);
	}

	// varlist for options (optional)
	if(m_Flags & ZPFLAG_STORE_OPTIONS) {
		zlist_CreateItem(hItem, VarID_varlist, NULL);
	}

	return zlist_nItems(m_hPointList)-1;
}
bool CzPointList::DeletePoint(long index) {
	return zlist_DelItemI(m_hPointList, index);
}
bool CzPointList::DeletePoints() {
	return zlist_DelAll(m_hPointList);

}
long CzPointList::FindPoint(const char* lpName) {
	CzList ls;
	if(!ls.Attach(m_hPointList)) {
		zeoReportError("CzPointList::FindPoint error:\r\n - cannot attach point list");
		return -1;
	}
	return ls.FindItemA(lpName);
}
const char* CzPointList::GetPointName(long index) {

	CzList ls;
	if(!ls.Attach(m_hPointList)) {
		zeoReportError("CzPointList::GetPointName error:\r\n - cannot attach point list");
		return NULL;
	}

	return ls.GetItemName(index);
}

// point data

bool CzPointList::SetPointXYZ(long index, vector3d* pv) {
	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;
	if(!pv)	return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 0); // pos is always at 0

	if(!zvar_SetValueEx(hSubItem, VarID_vector3d, pv)) {
		return false;
	}

	return true;
}
bool CzPointList::SetPointXYZ(long index, double dx, double dy, double dz) {
	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 0); // pos is always at 0

	vector3d v = VectorSet3d(dx, dy, dz);
	if(!zvar_SetValueEx(hSubItem, VarID_vector3d, &v)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointXYZ(long index, vector3d* pv) {
	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;
	if(!pv)		return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 0); // pos is always at 0

	if(!zvar_GetValueEx(hSubItem, VarID_vector3d, pv)) {
		return false;
	}

	return true;
}

bool CzPointList::GetPointXYZ(long index, double *pdx, double *pdy, double *pdz) {
	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 0); // pos is always at 0

	vector3d v;
	if(!zvar_GetValueEx(hSubItem, VarID_vector3d, &v)) {
		return false;
	}

	if(pdx) *pdx = v.x;
	if(pdy) *pdy = v.y;
	if(pdz) *pdz = v.z;

	return true;
}


bool CzPointList::SetPointYPR(long index, vector3d* pv) {
	if(!(m_Flags & ZPFLAG_STORE_YPR)) return false;
	if(!pv)	return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 1); // if present, ypr is always at 1

	if(!zvar_SetValueEx(hSubItem, VarID_vector3d, pv)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointYPR(long index, vector3d* pv) {
	if(!(m_Flags & ZPFLAG_STORE_YPR)) return false;
	if(!pv)	return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 1); // if present, ypr is always at 1

	if(!zvar_GetValueEx(hSubItem, VarID_vector3d, pv)) {
		return false;
	}

	return true;
}

bool CzPointList::SetPointYPR(long index, double yaw, double pitch, double roll) {
	if(!(m_Flags & ZPFLAG_STORE_YPR)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 1); // if present, ypr is always at 1

	vector3d v = VectorSet3d(yaw, pitch, roll);
	if(!zvar_SetValueEx(hSubItem, VarID_vector3d, &v)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointYPR(long index, double *pyaw, double *ppitch, double *proll) {
	if(!(m_Flags & ZPFLAG_STORE_YPR)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	ZLIST hSubItem = zlist_GetItemI(hPoint, 1); // if present, ypr is always at 1

	vector3d v;
	if(!zvar_GetValueEx(hSubItem, VarID_vector3d, &v)) {
		return false;
	}

	if(pyaw)	*pyaw = v.x;
	if(ppitch)	*ppitch = v.y;
	if(proll)	*proll = v.z;

	return true;
}

bool CzPointList::SetPointFlags(long index, unsigned long Flags) {
	if(!(m_Flags & ZPFLAG_STORE_FLAGS)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_SetValueEx(hSubItem, VarID_uint, &Flags)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointFlags(long index, unsigned long *pFlags) {
	if(!(m_Flags & ZPFLAG_STORE_FLAGS)) return false;
	if(!pFlags)	return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_GetValueEx(hSubItem, VarID_uint, pFlags)) {
		return false;
	}

	return true;
}

bool CzPointList::SetPointParamI(long index, long Param) {
	if(!(m_Flags & ZPFLAG_STORE_PARAMI)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)		pos++;
	if(m_Flags & ZPFLAG_STORE_FLAGS)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_SetValueEx(hSubItem, VarID_int, &Param)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointParamI(long index, long *lpParam) {
	if(!(m_Flags & ZPFLAG_STORE_PARAMI)) return false;
	if(!lpParam)	return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)		pos++;
	if(m_Flags & ZPFLAG_STORE_FLAGS)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_GetValueEx(hSubItem, VarID_int, lpParam)) {
		return false;
	}

	return true;
}

bool CzPointList::SetPointParamD(long index, double Param) {
	if(!(m_Flags & ZPFLAG_STORE_PARAMD)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)		pos++;
	if(m_Flags & ZPFLAG_STORE_FLAGS)	pos++;
	if(m_Flags & ZPFLAG_STORE_PARAMI)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_SetValueEx(hSubItem, VarID_double, &Param)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointParamD(long index, double *lpParam) {
	if(!(m_Flags & ZPFLAG_STORE_PARAMD)) return false;
	if(!lpParam)	return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)		pos++;
	if(m_Flags & ZPFLAG_STORE_FLAGS)	pos++;
	if(m_Flags & ZPFLAG_STORE_PARAMI)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_GetValueEx(hSubItem, VarID_double, lpParam)) {
		return false;
	}

	return true;
}

bool CzPointList::SetPointOptions(long index, ZLIST hOptions) {
	if(!(m_Flags & ZPFLAG_STORE_OPTIONS)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)		pos++;
	if(m_Flags & ZPFLAG_STORE_FLAGS)	pos++;
	if(m_Flags & ZPFLAG_STORE_PARAMI)	pos++;
	if(m_Flags & ZPFLAG_STORE_PARAMD)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_CopyValue(hSubItem, hOptions)) {
		return false;
	}

	return true;
}
bool CzPointList::GetPointOptions(long index, ZLIST hOptions) {
	if(!(m_Flags & ZPFLAG_STORE_OPTIONS)) return false;

	ZLIST hPoint = zlist_GetItemI(m_hPointList, index);
	if(!hPoint) return false;

	int pos = 1;
	if(m_Flags & ZPFLAG_STORE_YPR)		pos++;
	if(m_Flags & ZPFLAG_STORE_FLAGS)	pos++;
	if(m_Flags & ZPFLAG_STORE_PARAMI)	pos++;
	if(m_Flags & ZPFLAG_STORE_PARAMD)	pos++;

	ZLIST hSubItem = zlist_GetItemI(hPoint, pos);

	if(!zvar_CopyValue(hOptions, hSubItem)) {
		return false;
	}

	return true;
}

bool CzPointList::ConvertTo(CzPointList* pl) {

	ZMAP pHF = NULL;
	ZMAP pSrcMap = NULL;
	ZMAP pDestMap = NULL;

	unsigned long Flags2 = pl->GetFlags();

	// get coord flags
	unsigned long xyFlags = m_Flags & ZPFLAG_XY_COORD_MASKALL;
	unsigned long xyFlags2 = Flags2 & ZPFLAG_XY_COORD_MASKALL;

	unsigned long zFlags = m_Flags & ZPFLAG_Z_COORD_MASKALL;
	unsigned long zFlags2 = Flags2 & ZPFLAG_Z_COORD_MASKALL;

	unsigned long yprFlags = m_Flags & ZPFLAG_YPR_COORD_MASKALL;
	unsigned long yprFlags2 = Flags2 & ZPFLAG_YPR_COORD_MASKALL;

	// get heightfield if required
	if(xyFlags & ZPFLAG_XY_COORD_PX_HF || xyFlags2 & ZPFLAG_XY_COORD_PX_HF ||
   	   m_Flags & ZPFLAG_Z_COORD_ABOVE_SURFACE || Flags2 & ZPFLAG_Z_COORD_ABOVE_SURFACE) {

		pHF = zproj_GetMap("HF");
		if(!pHF) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get heightfield");
			return false;
		}
		if(MAP_Heightfield!=zmap_GetMapType(pHF)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - heightfield is not initialised");
			return false;
		}
	}

	// get named source map if required
	if(xyFlags & ZPFLAG_XY_COORD_PX_MAP) {
		ZLIST hOptions = GetOptions();
		ZSTR hMapName = zlist_GetItemA(hOptions, "MapName");
		if(!hMapName) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get source map name");
			return false;

		}

		pSrcMap = zproj_GetMap(zstr_GetText(hMapName));
		if(!pSrcMap) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get source map");
			return false;
		}
		if(0==zmap_GetMapType(pSrcMap)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - source map is uninitialised");
			return false;
		}
	}

	// get named dest map if required
	if(xyFlags2 & ZPFLAG_XY_COORD_PX_MAP) {
		ZLIST hOptions = pl->GetOptions();
		ZSTR hMapName = zlist_GetItemA(hOptions, "MapName");
		if(!hMapName) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get dest map name");
			return false;

		}

		pDestMap = zproj_GetMap(zstr_GetText(hMapName));
		if(!pDestMap) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get dest map");
			return false;
		}

		if(0==zmap_GetMapType(pDestMap)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - dest map is uninitialised");
			return false;
		}
	}

	if(xyFlags & ZPFLAG_XY_COORD_METRES) {
		if(pHF || pSrcMap) {
			// cool, we're already there; we've got a map to play with
		} else {
			// we need to find a map to get the horiz scale

			ZLIST hOptions = GetOptions();
			ZSTR hMapName = zlist_GetItemA(hOptions, "MapName");
			if(hMapName) { // try named map first
				pSrcMap = zproj_GetMap(zstr_GetText(hMapName));
				if(!pSrcMap) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get source map");
					return false;
				}
				if(0==zmap_GetMapType(pSrcMap)) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - source map is uninitialised");
					return false;
				}
			} else { // try HF as backup
				pHF = zproj_GetMap("HF");
				if(MAP_Heightfield!=zmap_GetMapType(pHF)) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - heightfield is uninitialised");
					return false;
				}
			}
		}
	}


	if(xyFlags2 & ZPFLAG_XY_COORD_METRES) {
		if(pHF || pDestMap) {
			// cool, we're already there; we've got a map to play with
		} else {
			// we need to find a map to get the horiz scale

			ZLIST hOptions = pl->GetOptions();
			ZSTR hMapName = zlist_GetItemA(hOptions, "MapName");
			if(hMapName) { // try named map first
				pDestMap = zproj_GetMap(zstr_GetText(hMapName));
				if(!pDestMap) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get dest map");
					return false;
				}
				if(0==zmap_GetMapType(pDestMap)) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - dest map is uninitialised");
					return false;
				}
			} else { // try HF as backup
				pHF = zproj_GetMap("HF");
				if(MAP_Heightfield!=zmap_GetMapType(pHF)) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - heightfield is uninitialised");
					return false;
				}
			}
		}
	}

	// factors to convert input coords to rel (0->1)
	double fx = 1.0;
	double fy = 1.0;

	// factors to convert input coords to output coords
	double fx2 = 1.0;
	double fy2 = 1.0;

	if(xyFlags!=xyFlags2) {
		//
		// firstly, convert source coords to relative
		//
		if(xyFlags & ZPFLAG_XY_COORD_REL) {
			// do nothing, coords are relative
		} else
		if(xyFlags & ZPFLAG_XY_COORD_PX_HF) {
			if(!pHF) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid heightfield handle");
				return false;
			}

			int nx = zmap_GetWidth(pHF);
			int ny = zmap_GetHeight(pHF);

			if(nx<=1 || ny<=1) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid heightfield size");
				return false;
			}

			fx = 1.0 / (double)(nx-1); // such that a value of nx-1 in input coords is converted to 1.0
			fy = 1.0 / (double)(ny-1); // such that a value of nx-1 in input coords is converted to 1.0
		} else
		if(xyFlags & ZPFLAG_XY_COORD_PX_MAP) {

			if(!pSrcMap) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid source map handle");
				return false;
			}

			int nx = zmap_GetWidth(pSrcMap);
			int ny = zmap_GetHeight(pSrcMap);

			if(nx<=1 || ny<=1) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid source map size");
				return false;
			}

			fx = 1.0 / (double)(nx-1); // such that a value of nx-1 in input coords is converted to 1.0
			fy = 1.0 / (double)(ny-1); // such that a value of nx-1 in input coords is converted to 1.0
		} else
		if(xyFlags & ZPFLAG_XY_COORD_METRES) {
			float HorizScale = 1.0f;
			if(pSrcMap) {
				HorizScale = zmap_GetHorizScale(pSrcMap);
			} else
			if(pHF) {
				HorizScale = zmap_GetHorizScale(pHF);
			} else {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get source horiz scale");
				return false;
			}

			if(HorizScale<=0.0f) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid source horiz scale");
				return false;
			}

			fx = 1.0 / (double)HorizScale;
			fy = 1.0 / (double)HorizScale;
		} else {
			zeoReportError("CzPointList::ConvertTo error:\r\n - unknown source XY coordinate mode");
			return false;
		}


		//
		// secondly, convert rel coords to dest
		//
		if(xyFlags2 & ZPFLAG_XY_COORD_REL) {
			// do nothing, coords are relative
		} else
		if(xyFlags2 & ZPFLAG_XY_COORD_PX_HF) {
			if(!pHF) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid heightfield handle");
				return false;
			}

			int nx = zmap_GetWidth(pHF);
			int ny = zmap_GetHeight(pHF);

			if(nx<=1 || ny<=1) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid heightfield size");
				return false;
			}

			fx2 = fx * (double)(nx - 1); // such that a value of 1.0 in input coords is converted to nx-1
			fy2 = fy * (double)(ny - 1); // such that a value of nx-1 in input coords is converted to 1.0
		} else
		if(xyFlags2 & ZPFLAG_XY_COORD_PX_MAP) {

			if(!pSrcMap) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid source map handle");
				return false;
			}

			int nx = zmap_GetWidth(pSrcMap);
			int ny = zmap_GetHeight(pSrcMap);

			if(nx<=1 || ny<=1) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid source map size");
				return false;
			}

			fx2 = fx * (double)(nx - 1); // such that a value of 1.0 in input coords is converted to nx-1
			fy2 = fy * (double)(ny - 1); // such that a value of nx-1 in input coords is converted to 1.0
		} else
		if(xyFlags2 & ZPFLAG_XY_COORD_METRES) {
			float HorizScale = 1.0f;
			if(pSrcMap) {
				HorizScale = zmap_GetHorizScale(pSrcMap);
			} else
			if(pHF) {
				HorizScale = zmap_GetHorizScale(pHF);
			} else {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get source horiz scale");
				return false;
			}

			if(HorizScale<=0.0f) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - invalid source horiz scale");
				return false;
			}

			fx2 = fx * (double)HorizScale;
			fy2 = fy * (double)HorizScale;
		} else {
			zeoReportError("CzPointList::ConvertTo error:\r\n - unknown source XY coordinate mode");
			return false;
		}
	}


	pl->DeletePoints();
	for(int i=0; i<nPoints(); i++) {

		// create output point
		int j = pl->CreatePoint(GetPointName(i));
		if(j==-1) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot create new point");
			return false;
		}

		// get input XYZ
		vector3d v;
		if(!GetPointXYZ(i, &v)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get point XYZ");
			return false;
		}

		// do z conversion, if necessary
		if(zFlags != zFlags2) {

			// convert input heights to metres above sealevel
			if(zFlags & ZPFLAG_Z_COORD_ABOVE_SURFACE) {

				ZMAP hMap = NULL;
				if(MAP_Heightfield==zmap_GetMapType(pSrcMap)) {
					hMap = pSrcMap;
				} else
				if(pHF) {
					hMap = pHF;
				} else {
					zeoReportError("CzPointList::ConvertTo error:\r\n - no heightfield given; cannot transform input Z coord");
					return false;
				}

				double hfx = v.x * fx * (double)(zmap_GetWidth(hMap)-1);
				double hfy = v.y * fy * (double)(zmap_GetWidth(hMap)-1);

				float fz;
				if(!zmap_LinInterp(hMap, hfx, hfy, &fz)) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get input pixel height");
					return false;
				}

				v.z += fz;
			}

			// convert metres above sealevel to output heights
			if(zFlags2 & ZPFLAG_Z_COORD_ABOVE_SURFACE) {

				ZMAP hMap = NULL;
				if(MAP_Heightfield==zmap_GetMapType(pDestMap)) {
					hMap = pDestMap;
				} else
				if(pHF) {
					hMap = pHF;
				} else {
					zeoReportError("CzPointList::ConvertTo error:\r\n - no heightfield given; cannot transform output Z coord");
					return false;
				}

				double hfx = v.x * fx * (double)(zmap_GetWidth(hMap)-1);
				double hfy = v.y * fy * (double)(zmap_GetWidth(hMap)-1);

				float fz;
				if(!zmap_LinInterp(hMap, hfx, hfy, &fz)) {
					zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get input pixel height");
					return false;
				}

				v.z -= fz;
			}
		}

		// apply X/Y point scaling
		v.x *= fx2;
		v.y *= fy2;

		// set output XYZ
		if(!pl->SetPointXYZ(j, &v)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot set point XYZ");
			return false;
		}

		// handle YPR
		if(m_Flags & ZPFLAG_STORE_YPR && pl->GetFlags() & ZPFLAG_STORE_YPR) {
			vector3d v2;
			if(!GetPointYPR(i, &v2)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get point YPR");
				return false;
			}

			// convert ypr coords
			if(yprFlags != yprFlags2) {

				// convert input coords to radians
				if(yprFlags & ZPFLAG_YPR_COORD_DEG) {
					v2 = VectorScale3d(v2, (float)(PI / 180.0f));
				}

				// convert radians to output coords
				if(yprFlags2 & ZPFLAG_YPR_COORD_DEG) {
					v2 = VectorScale3d(v2, (float)(180.0f / PI));
				}
			}

			if(!pl->SetPointYPR(j, &v2)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot set point YPR");
				return false;
			}
		}

		// handle flags
		if(m_Flags & ZPFLAG_STORE_FLAGS && pl->GetFlags() & ZPFLAG_STORE_FLAGS) {
			unsigned long ul;
			if(!GetPointFlags(i, &ul)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get point flags");
				return false;
			}

			if(!pl->SetPointFlags(j, ul)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot set point flags");
				return false;
			}
		}

		// handle parami
		if(m_Flags & ZPFLAG_STORE_PARAMI && pl->GetFlags() & ZPFLAG_STORE_PARAMI) {
			long i32;
			if(!GetPointParamI(i, &i32)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get point parami");
				return false;
			}

			if(!pl->SetPointParamI(j, i32)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot set point parami");
				return false;
			}
		}

		// handle paramd
		if(m_Flags & ZPFLAG_STORE_PARAMD && pl->GetFlags() & ZPFLAG_STORE_PARAMD) {
			double d;
			if(!GetPointParamD(i, &d)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get point paramd");
				return false;
			}

			if(!pl->SetPointParamD(j, d)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot set point paramd");
				return false;
			}
		}

		// handle options
		if(m_Flags & ZPFLAG_STORE_OPTIONS && pl->GetFlags() & ZPFLAG_STORE_OPTIONS) {
			CzList ls;
			if(!GetPointOptions(i, ls)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get point options");
				return false;
			}

			if(!pl->SetPointOptions(j, ls)) {
				zeoReportError("CzPointList::ConvertTo error:\r\n - cannot set point options");
				return false;
			}
		}
	}

	// apply to sublists
	pl->DeleteSubLists();
	for(int i=0; i<nSubLists(); i++) {
		CzPointList ls2;
		if(!GetSubList(i, ls2)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get input sublist");
			return false;
		}

		int j = pl->CreateSubList(ls2.GetName(), pl->GetFlags());
		if(j==-1) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot create sublist");
			return false;
		}

		CzPointList ls3;
		if(!pl->GetSubList(j, ls3)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot get output sublist");
			return false;
		}

		if(!ls2.ConvertTo(&ls3)) {
			zeoReportError("CzPointList::ConvertTo error:\r\n - cannot convert sublist");
			return false;
		}
	}

	return true;
}