#ifndef __ZPOINTLIST_INCLUDED__
#define __ZPOINTLIST_INCLUDED__

#pragma once

#include <Zeolite.h>
#include <zVar.h>

// pos flags values (set in sublist, for all points in list)--

#define ZPFLAG_STORE_YPR		0x00000001	// if present, YPR is created for each point in sublist
#define ZPFLAG_STORE_FLAGS		0x00000002	// if present, flags are created for each point in sublist
#define ZPFLAG_STORE_OPTIONS	0x00000004	// if present, options are created for each point in sublist
#define ZPFLAG_STORE_PARAMI		0x00000008	// if present, an int param is created for each point in sublist
#define ZPFLAG_STORE_PARAMD		0x00000010	// if present, an int param is created for each point in sublist
#define ZPFLAG_STORE_MASKALL	0x000000FF

#define	ZPFLAG_XY_COORD_REL			0x00000100 // default XY coord mode is fractional coordinates (0...1, with 0 at left side, 1 at nx-1 right); these coords are invariant to resampling or rescaling
#define ZPFLAG_XY_COORD_PX_HF		0x00000200 // pixels, relative to project HF
#define ZPFLAG_XY_COORD_PX_MAP		0x00000400 // pixels, relative to a project map specified in options under MapName
#define ZPFLAG_XY_COORD_METRES		0x00000800 // metres, relative to bottom/left corner of map
#define ZPFLAG_XY_COORD_MASKALL		0x0000FF00

#define ZPFLAG_Z_COORD_ABOVE_SEALEVEL	0x00010000 // height above sealevel in m (if not set, this is default)
#define ZPFLAG_Z_COORD_ABOVE_SURFACE	0x00020000 // metres above HF surface
#define ZPFLAG_Z_COORD_MASKALL			0x00FF0000

#define ZPFLAG_YPR_COORD_RAD		0x01000000 // radians  (if not set, this is default)
#define ZPFLAG_YPR_COORD_DEG		0x02000000 // degrees
#define ZPFLAG_YPR_COORD_MASKALL	0xFF000000


class CzPointList : 
	public CzVar
{

public:

	CzPointList(void);
	CzPointList(ZLIST hContainer);
	~CzPointList(void);

	bool Create(ZLIST hContainer = NULL, unsigned long Flags = 0);
	bool CreateShared(const char* lpVarName, unsigned long Flags = 0);

	bool Attach(ZLIST hPointList);
	bool AttachToProjectList();
	void Release();
	
	unsigned long GetFlags() { return m_Flags; }
	ZLIST GetOptions() { return m_hOptions; }
	
	// sublists

	long nSubLists();
	long CreateSubList(const char* lpName = NULL, unsigned long Flags = 0);
	bool GetSubList(long index, CzPointList& SubList);
	bool DeleteSubList(long index);
	bool DeleteSubLists();
	long FindSubList(const char* lpName);
	
	// points

	long nPoints();
	long CreatePoint(const char* lpName = NULL);
	bool DeletePoint(long index);
	bool DeletePoints();
	long FindPoint(const char* lpName);
	const char* GetPointName(long index);
	
	// point data

	bool SetPointXYZ(long index, double dx, double dy, double dz);
	bool GetPointXYZ(long index, double *pdx, double *pdy, double *pdz);
	bool SetPointXYZ(long index, vector3d* pv);
	bool GetPointXYZ(long index, vector3d* pv);

	bool SetPointYPR(long index, double yaw, double pitch, double roll);
	bool GetPointYPR(long index, double *pyaw, double *ppitch, double *proll);
	bool SetPointYPR(long index, vector3d* pv);
	bool GetPointYPR(long index, vector3d* pv);

	bool SetPointFlags(long index, unsigned long Flags);
	bool GetPointFlags(long index, unsigned long *pFlags);

	bool SetPointParamI(long index, long Param);
	bool GetPointParamI(long index, long *lpParam);

	bool SetPointParamD(long index, double Param);
	bool GetPointParamD(long index, double *lpParam);

	bool SetPointOptions(long index, ZLIST hOptions);
	bool GetPointOptions(long index, ZLIST hOptions);
	
	// converter function

	bool ConvertTo(CzPointList* pl);
	
private:

	unsigned long m_Version;
	ZLIST m_hPointList;
	ZLIST m_hChildList;
	unsigned long m_Flags;
	ZLIST m_hOptions;
};

#endif