#include "Zeolite.h"
#include "zvar.h"
#include <assert.h>

CzVar::CzVar(void) {
	m_hVar = 0;
	m_AutoDelete = false;
}
CzVar::CzVar(ZVAR hVar) {
	m_hVar = hVar;
	m_AutoDelete = false;
}
CzVar::CzVar(long VarID, const char* lpVarName) {
	m_hVar = 0;
	m_AutoDelete = false;

	if(lpVarName) {
		CreateShared(VarID, lpVarName);
	} else {
		Create(VarID);
	}
}

CzVar::~CzVar(void) {
	if(m_hVar!=NULL && m_AutoDelete) {
		if(!Delete()) {
			zeoReportError("CzVar::~CzVar error:\r\n - call to Delete failed");
		}
	}
}

bool CzVar::Attach(ZVAR hVar) {

	if(m_hVar && m_AutoDelete) {
		if(!Delete()) {
			zeoReportError("CzVar::Attach error:\r\n - call to Delete failed");
			return false;
		}
	}

	if(!hVar) {
#ifdef DEBUG
		zeoReportError("CzVar::Attach error:\r\n - invalid variable handle");
#else
		zeoWriteToLog("CzVar::Attach error:\r\n - invalid variable handle");
#endif
		return false;
	}
	
	m_hVar = hVar;
	m_AutoDelete = false;
	return true;
}

ZVAR CzVar::GetZVAR() { return m_hVar; }

bool CzVar::Create(long VarID) {
	
	if(m_hVar && m_AutoDelete) {
		if(!Delete()) {
			zeoReportError("CzVar::Create error:\r\n - call to Delete failed");
			return false;
		}
	}

	if(!(m_hVar = zvar_Create(VarID))) {
		zeoReportError("CzVar::Create error:\r\n - call to zvar_Create failed");
		return false;
	}

	m_AutoDelete = true;

	return true;
}
bool CzVar::CreateShared(long VarID, const char* lpVarName) {
	
	if(m_hVar && m_AutoDelete) {
		if(!Delete()) {
			zeoReportError("CzVar::CreateShared error:\r\n - call to Delete failed");
			return false;
		}
	}

	if(!(m_hVar = zvar_CreateShared(VarID, lpVarName))) {
		zeoReportError("CzVar::Create error:\r\n - call to zvar_CreateShared failed");
		return false;
	}

	m_AutoDelete = true;

	return true;
}

bool CzVar::GetSharedVar(const char* lpVarName) {
	if(m_hVar && m_AutoDelete) {
		if(!Delete()) {
			zeoReportError("CzVar::GetSharedVar error:\r\n - call to Delete failed");
			return false;
		}
	}

	if(!Attach(zvar_GetSharedVar(lpVarName))) {
		zeoReportError("CzVar::GetSharedVar error:\r\n - call to Attach failed");
		return false;
	}

	return true;
}

bool CzVar::Delete() {
	if(!m_hVar) {
		/* er, no, we're allowed to call Delete on an ininitialised variable (otherwise we'd always have to test)
#ifdef _DEBUG
		zeoReportError("CzVar::Delete error:\r\n - variable not initialised");
		assert(0); // lets track it down in debug build!
#else
		zeoWriteToLog("CzVar::Delete error:\r\n - variable not initialised");
#endif
		*/
		return true;
	}

	bool rval = false;
	if(!(rval = zvar_Delete(m_hVar))) {
#ifdef _DEBUG
		zeoReportError("CzVar::Delete error:\r\n - call to zvar_Delete failed");
		assert(0); // lets track it down in debug build!
#else
		zeoWriteToLog("CzVar::Delete error:\r\n - call to zvar_Delete failed");
#endif
	}
	
	m_hVar = 0;
	return rval;
}
void CzVar::Release() {
	m_hVar = 0;

	m_AutoDelete = false; // 2015 June 19 can't auto-delete if release is set
}

const char* CzVar::GetName() {
	if(!m_hVar)	return 0;

	return zvar_GetName(m_hVar);
}
bool CzVar::Rename(const char* lpNewName) {
	if(!m_hVar)	return false;

	return zvar_Rename(m_hVar, lpNewName);
}

long CzVar::GetTypeID() {
	if(!m_hVar)	return false;

	return zvar_GetTypeID(m_hVar);
}
bool CzVar::GetTypeName(ZSTR hTypeName) {
	if(!m_hVar)	return false;

	return ztype_GetName(GetTypeID(), hTypeName);
}
bool CzVar::IsType(long VarID) {
	if(!m_hVar) {
		if(VarID==VarID_INVALID)
				return true;
		else	return false;
	}

	return zvar_IsType(m_hVar, VarID);
}
bool CzVar::ReInit(long VarID) {
	if(!m_hVar)	{
		zeoReportError("CzVar::ReInit error:\r\n - variable not initialised");
		return false;
	}

	if(!zvar_ReInit(m_hVar, VarID)) {
		zeoReportError("CzVar::ReInit error:\r\n - call to zvar_ReInit failed");
		return false;
	}

	return true;
}

bool CzVar::GetValue(long VarID, void* pValue) {
	if(!m_hVar)	return false;

	return zvar_GetValueEx(m_hVar, VarID, pValue);
}
bool CzVar::SetValue(long VarID, void* pValue) {
	if(!m_hVar)	return false;

	return zvar_SetValueEx(m_hVar, VarID, pValue);
}
bool CzVar::GetValueText(ZSTR hStrVar) {
	if(!m_hVar)	return false;

	return zvar_GetValueText(m_hVar, hStrVar);
}
bool CzVar::SetValueText(const char* lpValueText) {
	if(!m_hVar)	return false;

	return zvar_SetValueText(m_hVar, lpValueText);
}

bool CzVar::CopyValue(ZVAR hVarSrc) {
	if(!m_hVar)	return false;

	return zvar_CopyValue(m_hVar, hVarSrc);
}

void* CzVar::GetDataPtr(long VarID) {
	if(!m_hVar)	return NULL;

	if(VarID!=GetTypeID()) {
		return NULL;
	}

	return zvar_GetDataPtr(m_hVar);
}

bool CzVar::SetVarRef(ZVAR hSrcVar) {
	if(!m_hVar) {
		zeoReportError("CzVar::SetVarRef error:\r\n - variable not initialised");
		return false;
	}

	if(!zvar_SetVarRef(m_hVar, hSrcVar)) {
		zeoReportError("CzVar::SetVarRef error:\r\n - call to zvar_SetVarRef failed");
		return false;
	}

	m_AutoDelete = false; // 2015 June 19 can't auto-delete if SetVarRef is set

	return true;
}

bool CzVar::EditUI(const char* lpPrompt, unsigned long Flags, ZLIST hOptions) {
	if(!m_hVar)	return false;

	return zvar_EditUI(m_hVar, lpPrompt, Flags, hOptions);
}