#pragma once
#include "zvar.h"
#include "zmaptile.h"

class CzMapTile;

class CzMap :
	public CzVar
{
public:
	CzMap(void);
	CzMap(ZMAP hMap);
	~CzMap(void);

	// object creation

	bool Create();
	bool CreateShared(const char* lpVarName);
	bool Attach(ZVAR hVar);
	void Release();

	// map initialisation

	bool	Init(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag);
	bool    Init2(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, unsigned long AllocMode = MAPALLOC_DONTCARE, long TileSize = 0);
	bool	InitMosaic(long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag, const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName);
	bool	Free();
	unsigned long GetAllocMode();

	// map size

	long	nx(); // get width
	long	ny(); // get height
	__int64 nPixels(); // width * height
	float	GetHorizScale();
	bool	SetHorizScale(float scale);
	long	GetTileSize();
	long	GetTileCountX();
	long	GetTileCountY();
	bool	GetMinMaxAlt(float& minval, float& maxval);
	bool	SetMinMaxAlt(float& minval, float& maxval);

	// pixel access

	bool	GetPixel(long x, long y, void* pValue);
	bool	GetPixel(__int64 k, void* pValue); // linearised index
	bool	SetPixel(long x, long y, void* pValue);
	bool	SetPixel(__int64 k, void* pValue);  // linearised index
	bool	LinInterp(double dx, double dy, void* pValue);

	// settings and flags

	long	GetMapType();
	long	GetPixelSize();
	bool	GetWrapFlag();
	bool	SetWrapFlag(bool WrapFlag);
	bool	GetBusyFlag();
	bool	SetBusyFlag(bool BusyFlag);
	bool	GetMosaicFlag();
	bool	GetFlag(long FlagID);
	bool	SetFlag(long FlagID, bool FlagValue);
	ZLIST	GetSettingsList();
	bool	GetNameInProject(ZVAR hStr);

	// file input and output

	bool	SaveFile(const char* lpFileName, ZFORMAT hFormat, bool SetFlags, bool ShowProgress);
	bool	LoadFile(const char* lpFileName, long MapTypeID, ZFORMAT hFormat, bool SetFlags, bool ShowProgress);
	bool	LoadMosaic(const char* lpMosaicFileName);
	bool	SaveMosaic();
	bool	SaveMosaicAs(const char* lpFileName);
	ZFORMAT GetFormat();
	bool	SetFormat(ZFORMAT hFormat);
	const char*	GetFilename();
	bool	ExportMap(const char* lpFileName, ZFORMAT hFormat, long nx, long ny);
	bool	ExportMosaic(const char* lpFileName, ZFORMAT hFormat, long nx, long ny, long TileSize, const char* lpProjMapName);

	// mipmaps

	bool	GenMipmaps(long ResStep, long MaxLevel, long TileSize);
	bool	ClearMipmaps();
	ZMAP	GetMipmapLevel(long MipLevel);
	long	GetMipmapResStep();
	long	GetMipmapMaxLevel();
	long	nMipmaps();
	bool	AllocMipmaps(long MipIncr, long MaxMip);
	bool	GetMipmapLevel2(long MipID, long &MipRes, ZMAP* ppMipMap);

	// mosaic functions

	bool	CombineMosaic();
	bool	SplitToMosaic(const char* lpMosaicFileName, long TileSize, ZFORMAT hFormat, const char* lpProjMapName);
	bool	GetMosaicTile(long ti, long tj, CzMapTile& tile);

	// direct memory access

	void*	GetDataPtr();
	void*	GetScanlinePtr(long j);
	void**	GetScanlineArray();
	void*	GetAt(long x, long y);
	bool	IsContiguous();
	bool	MakeContiguous();
	bool	AsContiguousMap(CzMap& dest);
	bool	OpenDirectInterface();
	void	CloseDirectInterface();

protected:

	// direct interface variables
	bool	m_DirectInterfaceFlag;
	long	m_nx;
	long	m_ny;
	long	m_nTilesX;
	long	m_nTilesY;
	__int64 m_nPixels;
	long	m_TileSize;
	long	m_PixelMemSize;
	long	m_AllocMode;
	void*	m_pMem;
	bool	m_Contiguous;

};
